
function collectAllComments(to, history) {
    history = history || [];

    var comment = "";
    if (to.name !== undefined && to.name !== null) {
        comment += to.name;
    }
    if (to.comment !== undefined && to.comment !== null) {
        comment += (comment === "" ? "" : ": ") + $("<div>" + to.comment + "</div>").text();
    }
    if (to.fullComment !== undefined && to.fullComment !== null) {
        comment += "\n" + $("<div>" + to.fullComment + "</div>").text();
    }

    switch (to.typeValue.type) {
        case "simple":
            break;
        case "enum":
            _.each(to.typeValue.values, function (value) {
                comment += "\n" + value.name + (value.comment !== null ? ": " + value.comment : "");
            });
            break;
        case "collection":
            comment += "\n" + collectAllComments(to.typeValue, history);
            break;
        case "map":
            comment += "\n" + collectAllComments(to.typeValue, history);
            break;
        case "abstract":
            if (_.indexOf(history, to.typeValue.name) === -1) {
                var newHistory = history.slice(0); // clone the history
                newHistory.push(to.typeValue.name);
                _.each(to.typeValue.subTypes, function (subType) {
                    _.each(subType.to.content, function (field) {
                        comment += "\n" + collectAllComments(field, newHistory);
                    });
                });
            }
            break;
        case "complex":
            if (_.indexOf(history, to.typeValue.name) === -1) {
                var newHistory = history.slice(0); // clone the history
                newHistory.push(to.typeValue.name);
                _.each(to.typeValue.content, function (field) {
                    comment += "\n" + collectAllComments(field, newHistory);
                });
            }
            break;
    }
    return comment;
}

var searchDocs = [];
var searchIdx = null;
var searchResults = null;

function createSearchIndex() {

    // Boost values have no effect since in the view the interfaces are ordered by url
    searchIdx = lunr(function () {
        this.field('url', { boost: 1 });
        this.field('title', { boost: 1 });
        this.field('comment', { boost: 1 });
        this.field('body', { boost: 1 });
        this.field('output', { boost: 1 });
        this.field('params', { boost: 1 });
        this.field('statusCodes', { boost: 1 });

        // Removes all the token modifiers
        this.pipeline.reset();
        // Add the stemmer, this filter changes words to their base (eg. searched becomes search)
        this.pipeline.add(lunr.stemmer);
        // Add the stopWordFilter, this filter removes common words from the search tokens
        //this.pipeline.add(lunr.stopWordFilter);

        var idx = 0;

        _.each(com.qmino.miredot.restApiSource.interfaces, function (iface) {
            var doc = {
                id: idx,
                iface: iface,
                url: iface.url.replace(/\//g, " "),
                title: iface.title,
                comment: $("<div>" + iface.beschrijving + "</div>").text(),
                body: "",
                output: "",
                params: "",
                statusCodes: ""
            };
            ++idx;

            _.each(iface.inputs.PATH, function (param) {
                doc.params += collectAllComments(param) + "\n";
            });
            _.each(iface.inputs.QUERY, function (param) {
                doc.params += collectAllComments(param) + "\n";
            });
            _.each(iface.inputs.HEADER, function (param) {
                doc.params += collectAllComments(param) + "\n";
            });
            _.each(iface.inputs.COOKIE, function (param) {
                doc.params += collectAllComments(param) + "\n";
            });
            _.each(iface.inputs.MATRIX, function (param) {
                doc.params += collectAllComments(param) + "\n";
            });
            _.each(iface.inputs.BODY, function (param) {
                doc.body += collectAllComments(param) + "\n";
            });
            if (iface.output.typeValue !== undefined) {
                doc.output += collectAllComments(iface.output);
            }
            _.each(iface.statusCodes, function(statusCode) {
                doc.statusCodes += statusCode.httpCode;
                if (statusCode.comment !== null && statusCode.comment !== undefined) {
                    doc.statusCodes += ": " + statusCode.comment;
                }
                doc.statusCodes += "\n";
            });
            searchDocs.push(doc);
        });

        var self = this;
        _.each(searchDocs, function(doc) {
            self.add(doc);
        });
    });

}
