'use strict';

/* Controllers */


/**
 * The main viewmodel for the doc. Loads a variable named restApiSource.
 * @param $scope
 * @constructor
 */
miredotModule.controller('DocRoot', ["$scope", "$location", "$localStorage", "$filter", "$http", "$anchorScroll", "$timeout",
function DocRoot($scope, $location, $localStorage, $filter, $http, $anchorScroll, $timeout) {

    $scope.restBase = 'http://www.miredot.com/miredot/rest/';
    $scope.visitWebsiteForProVersion = 'Visit our <a href="http://www.miredot.com/price/?licenserequest=pro" target="_blank">website</a> to get the full version (free for open source).';

    $scope.projectTitle = com.qmino.miredot.restApiSource.projectTitle;
    $scope.miredotVersion = com.qmino.miredot.restApiSource.miredotVersion;
    $scope.validLicense = com.qmino.miredot.restApiSource.validLicense;
    $scope.licenseType = com.qmino.miredot.restApiSource.licenseType;
    $scope.licenseErrorMessage = "";
    $scope.licenseHash = com.qmino.miredot.restApiSource.licenseHash;
    $scope.hideSidebar = com.qmino.miredot.restApiSource.hideSidebar;
    $scope.allowUsageTracking = com.qmino.miredot.restApiSource.allowUsageTracking;
    $scope.dateOfGeneration = com.qmino.miredot.restApiSource.dateOfGeneration;
    $scope.issuesTabHidden = com.qmino.miredot.restApiSource.issuesTabHidden;
    $scope.singlePage = com.qmino.miredot.restApiSource.singlePage;
    $scope.hideLogoOnTop = com.qmino.miredot.restApiSource.hideLogoOnTop;


    if (com.qmino.miredot.restApiSource.licenseErrorMessage !== null
        && com.qmino.miredot.restApiSource.licenseErrorMessage !== undefined) {
        for (var i = 0; i < com.qmino.miredot.restApiSource.licenseErrorMessage.length; i++) {
            $scope.licenseErrorMessage += com.qmino.miredot.restApiSource.licenseErrorMessage[i] + '<br>';
        }

        if (com.qmino.miredot.restApiSource.onlineLicense) {
            $scope.licenseErrorMessage += ' <a href="' + com.qmino.miredot.restApiSource.loginUrl
                + '">Login</a> to upgrade your subscription.';
        }
    }


    $scope.$storage = $localStorage.$default({
        globalCollapsedState: false
    });
    $scope.baseUrl = {
        value: (function () {
            var baseUrl = "http://example.com";
            if ($scope.validLicense && $scope.licenseType == 'PRO') {
                baseUrl = $location.search().baseUrl || com.qmino.miredot.restApiSource.baseUrl || baseUrl;
            }
            return baseUrl;
        })()
    };
    $scope.applicationPath = com.qmino.miredot.restApiSource.applicationPath;

    $scope.editingBaseUrl = false;
    $scope.projectWarnings = com.qmino.miredot.projectWarnings;
    $scope.interfaces = com.qmino.miredot.restApiSource.interfaces;
    $scope.tos = com.qmino.miredot.restApiSource.tos;
    $scope.processErrors = com.qmino.miredot.processErrors;
    $scope.jsonDocConfig = {
        enabled: com.qmino.miredot.restApiSource.jsonDocEnabled,
        hidden: com.qmino.miredot.restApiSource.jsonDocHidden
    };

    $scope.searchByExample = "";
    $scope.fullTextSearch = "";
    $scope.searchQuery = {url: "", http: ""};
    $scope.location = $location;
    $scope.navigationView = 'hierarchical';

    $scope.statusCodes = {};
    $scope.statusCodes["100"] = "Continue";
    $scope.statusCodes["101"] = "Switching Protocols";
    $scope.statusCodes["200"] = "OK";
    $scope.statusCodes["201"] = "Created";
    $scope.statusCodes["202"] = "Accepted";
    $scope.statusCodes["203"] = "Non-Authoritative Information";
    $scope.statusCodes["204"] = "No Content";
    $scope.statusCodes["205"] = "Reset Content";
    $scope.statusCodes["206"] = "Partial Content";
    $scope.statusCodes["300"] = "Multiple Choices";
    $scope.statusCodes["301"] = "Moved Permanently";
    $scope.statusCodes["302"] = "Found";
    $scope.statusCodes["303"] = "See Other";
    $scope.statusCodes["304"] = "Not Modified";
    $scope.statusCodes["305"] = "Use Proxy";
    $scope.statusCodes["306"] = "Switch Proxy";
    $scope.statusCodes["307"] = "Temporary Redirect";
    $scope.statusCodes["308"] = "Permanent Redirect";
    $scope.statusCodes["400"] = "Bad Request";
    $scope.statusCodes["401"] = "Unauthorized";
    $scope.statusCodes["402"] = "Payment Required";
    $scope.statusCodes["403"] = "Forbidden";
    $scope.statusCodes["404"] = "Not Found";
    $scope.statusCodes["405"] = "Method Not Allowed";
    $scope.statusCodes["406"] = "Not Acceptable";
    $scope.statusCodes["407"] = "Proxy Authentication Required";
    $scope.statusCodes["408"] = "Request Timeout";
    $scope.statusCodes["409"] = "Conflict";
    $scope.statusCodes["410"] = "Gone";
    $scope.statusCodes["411"] = "Length Required";
    $scope.statusCodes["412"] = "Precondition Failed";
    $scope.statusCodes["413"] = "Request Entity Too Large";
    $scope.statusCodes["414"] = "Request-URI Too Long";
    $scope.statusCodes["415"] = "Unsupported Media Type";
    $scope.statusCodes["416"] = "Requested Range Not Satisfiable";
    $scope.statusCodes["417"] = "Expectation Failed";
    $scope.statusCodes["418"] = "I'm a teapot";
    $scope.statusCodes["422"] = "Unprocessable Entity";
    $scope.statusCodes["423"] = "Locked";
    $scope.statusCodes["424"] = "Failed Dependency";
    $scope.statusCodes["426"] = "Upgrade Required";
    $scope.statusCodes["428"] = "Precondition Required";
    $scope.statusCodes["429"] = "Too Many Requests";
    $scope.statusCodes["431"] = "Request Header Fields Too Large";
    $scope.statusCodes["500"] = "Internal Server Error";
    $scope.statusCodes["501"] = "Not Implemented";
    $scope.statusCodes["502"] = "Bad Gateway";
    $scope.statusCodes["503"] = "Service Unavailable";
    $scope.statusCodes["504"] = "Gateway Timeout";
    $scope.statusCodes["505"] = "HTTP Version Not Supported";

    ensureLinksHaveTargetInApiIntro();

    setGlobalCollapsedState($localStorage.globalCollapsedState);
    $scope.hierarchyOpen = true;

    // Watch the location hash and update the view when it changes.
    $scope.$watch('location.hash()', function (newValue, oldValue) {
        if (oldValue != newValue) {
            setView(newValue);
        }
    });

    //$http.jsonp(
    //    $scope.restBase +
    //    'version' +
    //    '?hash=' + $scope.licenseHash + '&version=' + $scope.miredotVersion + '&licensetype=' + ($scope.licenseType || 'FREE') +
    //    '&callback=JSON_CALLBACK' + '&noi=' + $scope.interfaces.length)
    //    .success(function (data) {
    //        if (data.upToDate) {
    //            $scope.versionCheckResult = "";
    //        } else {
    //            $scope.versionCheckResult = " | New version available: " + data.version;
    //        }
    //    });

    $scope.isSearchIndexCreated = false;
    $scope.navigationRightNavigateTo = function(view) {
        $scope.navigationView = view;
        setTimeout(function() {
            if($scope.navigationView === 'search' && searchIdx == null) {
                createSearchIndex();
                $scope.$apply(function() {
                    $scope.isSearchIndexCreated = true;
                });
            }
        }, 1);
    };

    $scope.formatTypeValue = function (typeValue) {
        switch (typeValue.type) {
            case 'collection':
                return '[ ' + $scope.formatTypeValue(typeValue.typeValue) + ' ]';

            case 'enum':
                return 'enum';

            default:
                return typeValue.typeValue;
        }
    };

    $scope.formatDefaultValue = function (typeValue, defaultValue) {
        switch (typeValue.type) {
            case 'enum':
                // TODO CHANGE
                var enumValues = enumArrayToString(typeValue);
                enumValues = enumValues.replace(defaultValue, '<span class="default" title="Default value">' + defaultValue + '</span>');
                return enumValues + enumComment(typeValue);
            default:
                if (defaultValue != undefined) {
                    return '<span class="default" title="Default value">' + defaultValue + '</span>'
                }
                return "";
        }
    };

    /**
     * Sets $scope.view, based on the given hash.
     * Currently sets the view to 'interfaces' when it's not 'warnings'.
     * eg. #1465486435 (interface hash) will cause the view to be 'interfaces'
     *
     * @param hash A string from which the current view will be determined
     */
    function setView(hash) {
        if (hash === 'warnings') {
            $scope.view = 'warnings';
        } else {
            if (/^-?\d+$/.test(hash)) {
                if ($scope.getInterfaceByHash(hash) === null || $scope.getInterfaceByHash(hash) === undefined) {
                    window.location.hash = '';
                } else {
                    $scope.openResource($scope.getInterfaceByHash(hash).url, hash);
                }
            }
            $scope.view = 'interfaces';
        }
    }

    $scope.toggleJsonDoc = function (anchor) {
        $scope.jsonDocConfig.hidden = !$scope.jsonDocConfig.hidden;
        //scroll after digest
        $timeout(function () {
            $location.hash(anchor);
            $anchorScroll();
        });
    };

    /**
     * Ensures that all links in the API intro have a target.
     */
    function ensureLinksHaveTargetInApiIntro() {
        $("#intro a").attr('target', function (i, current) {
            return current || '_self';
        });
    }

    /**
     * Function to clear the selection of service tags
     */
    function clearServiceTagSelection() {
        _.each($scope.serviceTags, function (serviceTag) {
            serviceTag.selected = false;
        });
    }

    // Build a map interfaceHash->interfaceObject
    var interfacesByHash = {};
    _.each($scope.interfaces, function (iface) {
        interfacesByHash[iface.hash] = iface;
    });

    /**
     * Build a list of tag objects
     * {
     *   name: string, tag name
     *   selected: boolean, selected for filtering?
     * }
     */
    $scope.serviceTags = (function () {
        var tagNames = [];
        _.each($scope.interfaces, function (currentInterface) {
            _.each(currentInterface.tags, function (tagName) {
                tagNames.push(tagName);
            });
        });
        tagNames = _.uniq(tagNames);
        return _.map(tagNames, function (tagName) {
            return {name: tagName, selected: false};
        });
    }());

    /**
     * Function to look up the state of a service tag by name
     * @param tagName The name of the service tag
     */
    $scope.isServiceTagSelected = function (tagName) {
        var tag = _.find($scope.serviceTags, function (serviceTag) {
            return serviceTag.name === tagName;
        });
        return tag.selected;
    };

    // Build a map warningCategory->[warning]
    $scope.projectWarningsByType = (function () {
        var result = {};
        _.each($scope.projectWarnings, function (projectWarning) {
            result[projectWarning.category] = result[projectWarning.category] || [];
            result[projectWarning.category].push(projectWarning);
        });
        return result;
    }());

    function appendUrl(rootParts, url, method, hash, rootResource) {

        var currentResource = null;
        var parentResource = rootResource;
        _.each(rootParts, function (rootPart) {

            if (rootPart != "") {
                currentResource = _.find(parentResource.resources, function (resource) {
                    return resource.name === rootPart
                });

                if (!currentResource) {
                    currentResource = {
                        name: rootPart,
                        resources: [],
                        leafResources: []
                    };
                    parentResource.resources.push(currentResource);
                }
                parentResource = currentResource;
            } else {
                currentResource = parentResource;
            }
        });

        if(currentResource !== null && currentResource !== undefined){
            if(currentResource.leafResources !== null && currentResource.leafResources !== null){
                var existingLeaf = _.find(currentResource.leafResources, function (leaf) {
                    return leaf.url === url;
                });
            }
        }

        if (existingLeaf) {
            existingLeaf.methods.push({method: method, hash: hash});
        } else {
            if(currentResource !== null && currentResource !== undefined){
                currentResource.leafResources.push({
                    name: '',
                    url: url,
                    methods: [{method: method, hash: hash}]
                });
            }
        }
    }

    /**
     * Splits baseUrl in chunks on '/', but takes regexes into account (ie: does not split on a '/' inside a regex)
     * @param baseUrl a string to split
     * @returns {Array} an array of strings containing the different chunks
     */
    function splitPath(baseUrl) {
        var result = [];
        var buffer = "";
        var openBraces = 0;
        var closedBraces = 0;

        for (var i = 0; i < baseUrl.length; i++) {
            var char = baseUrl.charAt(i);
            if (char == "{") openBraces++;
            if (char == "}") closedBraces++;
            if (char == "/" && openBraces == closedBraces) {
                if (buffer != "") {
                    result.push(buffer);
                }
                buffer = "";
                openBraces = 0;
                closedBraces = 0;
            } else {
                buffer += char;
            }
        }
        if (buffer != "") {
            result.push(buffer);
        }
        return result;
    }

    /**
     * Constructs a tree structure of the available resources.
     * {
     *    '/rest' :{
     *       resource: '/rest',
     *       subresources: {
     *          '/myresource': {
     *             resource: '/myresource',
     *             subresources: [ .. ]
     *          },
     *          {
     *             ..
     *          }
     *       }
     *    }
     * }
     */
    function splitPaths(interfaces) {
        //add the root resource
        var resources = [
            {
                name: "",
                resources: [],
                leafResources: []
            }
        ];
        _.each(interfaces, function (element, index, list) {
            if (element.url.charAt(0) !== '/') {
                element.url = '/' + element.url;
            }
            if (element.url.charAt(element.url.length - 1) !== "/") {
                element.url += "/";
            }

            var baseUrl = element.url;
            var result = ["/"];

            if(baseUrl !== "/"){
                result = splitPath(baseUrl);
            }

            appendUrl(result, element.url, element.http, element.hash, resources[0]);
        });

        //compact empty middle resources: like IntelliJ IDEA's "Compact Empty Middle Packages"
        _.each(resources, function (subResource, index, list) {
            compactEmptyMiddleResources(subResource.resources, subResource.leafResources);
        });

        //remove the root resource if not used
        if (resources[0].leafResources.length === 0) {
            resources = resources[0].resources;
        }

        //sort resources on highest level
        resources.sort(function (r1, r2) {
            return r1.name.localeCompare(r2.name)
        });

        return resources;
    }

    function compactEmptyMiddleResources(resources, leafResources) {
        var subResourcesToRemove = [];
        _.each(resources, function (subResource, index, list) {
            var foundLeafResult = findSolitaryLeaf(subResource, "");
            if (foundLeafResult) {
                foundLeafResult.leaf.name = foundLeafResult.aggregatedName;
                leafResources.push(foundLeafResult.leaf);
                subResourcesToRemove.push(index);
            } else {
                compactEmptyMiddleResources(subResource.resources, subResource.leafResources);
            }
        });
        //remove sub resources after iterating: splice from highest index to lowest
        for (var i = subResourcesToRemove.length - 1; i >= 0; i--) {
            resources.splice(subResourcesToRemove[i], 1);
        }
    }

    /**
     * Searches and returns a solitary leaf with it's aggregated name.
     * Returns false if the given resource contains more than 1 leaf.
     *
     * returns {
     *             leaf: //the solitary leaf,
     *             aggregatedName: //the aggregated name of the solitary leaf
     *         }
     * @param subResource the sub resource to scan for solitary leafs (recursive)
     * @param name the aggregated leaf name (path)
     */
    function findSolitaryLeaf(subResource, name) {
        if (subResource.resources.length + subResource.leafResources.length !== 1) {
            return false; //we need exactly 1
        }
        var aggregatedName = concatenateWithSlash(name, subResource.name);
        if (subResource.leafResources.length === 1) {
            return {
                leaf: subResource.leafResources[0],
                aggregatedName: concatenateWithSlash(aggregatedName, subResource.leafResources[0].name)
            };
        } else {
            return findSolitaryLeaf(subResource.resources[0], aggregatedName);
        }
    }

    function concatenateWithSlash(firstPart, secondPart) {
        var result = '';
        if (firstPart) {
            result += firstPart;
        }
        if (firstPart && secondPart) {
            result += '/';
        }
        if (secondPart) {
            result += secondPart;
        }
        return result;
    }

    $scope.resourceTree = splitPaths(com.qmino.miredot.restApiSource.interfaces);

    $scope.getFirstLeaf = function (resource) {
        var orderBy = $filter('orderBy');
        if (resource.leafResources.length > 0) {
            var orderedLeafResources = orderBy(resource.leafResources, 'url');
            return orderedLeafResources[0];
        } else {
            return $scope.getFirstLeaf(orderBy(resource.resources, 'name')[0]);
        }
    };

    $scope.openResource = function (url, hash) {
        $scope.currentResource = findParent(url);
        window.location = '#' + hash;
    };

    if ($scope.interfaces !== undefined && $scope.interfaces !== null && $scope.interfaces.length > 0) {
        $scope.currentResource = findParent($scope.getFirstLeaf($scope.resourceTree[0]).url);
    } else {
        $scope.currentResource = null;
    }

    $scope.currentResourceSet = function () {
        if ($scope.singlePage) {
            return $scope.interfaces;
        }
        return _.filter($scope.interfaces, function (el) {
            return _.find($scope.currentResource.leafResources, function (leaf) {
                return leaf.url == el.url;
            });
        });
    };

    $scope.isResourceVisible = function (url) {
        if ($scope.singlePage) {
            return false;
        }
        return _.find($scope.currentResource.leafResources, function (leaf) {
            return leaf.url === url;
        });
    };

    function isResourcePartVisible(resource) {
        if ($scope.singlePage) {
            return false;
        }
        if (resource === $scope.currentResource) {
            return true;
        } else {
            return _.find(resource.resources, function (sub) {
                    return isResourcePartVisible(sub);
                }) !== undefined;
        }
    }

    function findParent(url) {
        return findParentRecursive($scope.resourceTree, url);
    }

    function findParentRecursive(resources, url) {
        for (var i = 0; i < resources.length; ++i) {
            var resource = resources[i];

            if (resource.leafResources.length !== 0) {
                if (_.find(resource.leafResources, function (leaf) {
                        return leaf.url === url
                    }) !== undefined) {
                    return resource;
                }
            }

            if (resource.resources.length !== 0) {
                var result = findParentRecursive(resource.resources, url);
                if (result !== null) {
                    return result;
                }
            }
        }
        return null;
    }

    function splitUrl(url) {
        return _.filter(url.split("/"), function (part) {
            return part !== "";
        });
    }

    $scope.isComplexObject = function (type) {
        return angular.isObject(type);
    };

    /**
     * Gives the interface (from $scope.interfaces) given the interface's hash ($scope.interfaces[x].hash).
     *
     * @param {string} interfaceHash The hash value of the interface
     * @return {object} An object contained in $scope.interfaces or undefined if not found.
     */
    $scope.getInterfaceByHash = function (interfaceHash) {
        return interfacesByHash[interfaceHash];
    };

    $scope.interfaceHttpOrderFunction = function (iface) {
        return _.indexOf($scope.httpMethods, iface.http);
    };

    $scope.methodHttpOrderFunction = function (method) {
        return _.indexOf($scope.httpMethods, method.method);
    };

    $scope.httpMethods = ['GET', 'HEAD', 'PUT', 'POST', 'DELETE', 'OPTIONS', 'PATCH'];

    _.each(customHttpMethods, function (httpMethod) {
        $scope.httpMethods.push(httpMethod);
    });

    $scope.toggleSearchQueryHttp = function (http) {
        if ($scope.searchQuery.http === http) {
            $scope.searchQuery.http = "";
        } else {
            $scope.searchQuery.http = http;
        }
    };

    $scope.setGlobalCollapsedState = setGlobalCollapsedState;

    function setGlobalCollapsedState(collapsed) {
        $localStorage.globalCollapsedState = collapsed;
        _.each($scope.interfaces, function (currentInterface) {
            currentInterface.collapsed = collapsed;
        })
    }

    $scope.collapseTree = function () {
        collapseResources($scope.resourceTree, false);
    };

    $scope.expandTree = function () {
        collapseResources($scope.resourceTree, true);
    };

    function collapseResources(resources, open, stopLevel) {
        if (stopLevel === undefined || stopLevel === null) {
            _.each(resources, function (resource) {
                resource.hierarchyOpen = open;
                collapseResources(resource.resources, open);
            })
        } else if (stopLevel < com.qmino.miredot.restApiSource.initialCollapseLevel) {
            _.each(resources, function (resource) {
                resource.hierarchyOpen = open;
                collapseResources(resource.resources, open, stopLevel + 1);
            })
        } else if (stopLevel !== undefined) {
            _.each(resources, function (resource) {
                if (isResourcePartVisible(resource)) {
                    resource.hierarchyOpen = true;
                    collapseResources(resource.resources, open, stopLevel + 1);
                }
            })
        }
    }

    function countResources(resources, leafResources) {
        var count = 0;
        if (leafResources) {
            count += leafResources.length;
        }
        if (resources) {
            count += resources.length;
            _.each(resources, function (resource) {
                count += countResources(resource.resources, resource.leafResources);
            });
        }
        return count;
    }

    // Set the view, based on the current location hash.
    // This must be called after $scope.openResource and $scope.getInterfaceByHash are defined
    // but before collapseResources is called
    setView($location.hash());

    if (com.qmino.miredot.restApiSource.initialCollapseLevel === undefined || com.qmino.miredot.restApiSource.initialCollapseLevel === null) {
        if (countResources($scope.resourceTree, null) > 35) {
            $scope.collapseTree();
            collapseResources($scope.resourceTree, false, 0);
        } else {
            $scope.expandTree();
            collapseResources($scope.resourceTree, true, 0);
        }
    } else {
        $scope.collapseTree();
        collapseResources($scope.resourceTree, true, 0);
    }

    $scope.doFullTextSearch = function () {
        if ($scope.fullTextSearch === '') {
            searchResults = null;
        } else {
            searchResults = [];
            var results = searchIdx.search($scope.fullTextSearch);
            _.each(results, function (result) {
                searchResults.push(searchDocs[result.ref].iface);
            });
        }
    }
}]);

/**
 * Parses an escaped url query string into key-value pairs.
 * @source AngularJS
 * @returns Object.<(string|boolean)>
 */
function parseKeyValue(/**string*/keyValue) {
    var obj = {}, key_value, key;
    angular.forEach((keyValue || "").split('&'), function (keyValue) {
        if (keyValue) {
            key_value = keyValue.split('=');
            key = decodeURIComponent(key_value[0]);
            obj[key] = angular.isDefined(key_value[1]) ? decodeURIComponent(key_value[1]) : true;
        }
    });
    return obj;
}

/**
 * Assumes as input an enum type with enums "a" and "b" and generates a string representation '"a" | "b"' taking
 * into account json value mappings (@JsonValue).
 *
 * @param enumType the enum type.
 * @returns {string} the stringrepresentation of the given enumType.
 */
function enumArrayToString(enumType) {
    var enumArray = [];
    if (enumType.jsonValueMappings !== null && enumType.jsonValueMappings !== undefined) {
        _.forEach(enumType.values, function (value) {
            enumArray.push({"name": enumType.jsonValueMappings[value.name]})
        });
    } else {
        enumArray = enumType.values;
    }
    var output = '';
    for (var i = 0; i < enumArray.length; i++) {
        if (i != 0) {
            output += ' | ';
        }
        output += enumArray[i].name;
    }
    return output;
}

var counter = 0;

function escapeHtml(value) {
    return value.toString().replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
}

function getNextEnumId() {
    return 'enum_' + (counter++);
}

function enumComment(enumType) {
    var enumArray = enumType.values;
    var comment = '';
    if (enumType.jsonValueComment !== null && enumType.jsonValueComment !== undefined
        && enumType.jsonValueComment !== "") {
        comment += enumType.jsonValueComment + '<br/>';
    }
    comment += "<ul>";
    _.each(enumArray, function (value) {
        if (value.comment !== null) {
            if (enumType.jsonValueMappings !== undefined && enumType.jsonValueMappings[value.name] !== null && enumType.jsonValueMappings[value.name] !== undefined
                && enumType.jsonValueMappings[value.name] !== "") {
                comment += '<li><strong>' + enumType.jsonValueMappings[value.name] + ':</strong> ' + value.comment + '<br/></li>';
            } else {
                comment += '<li><strong>' + value.name + ':</strong> ' + value.comment + '<br/></li>';
            }
        }
    });
    comment += "</ul>";
    if (comment === '<ul></ul>') {
        return '';
    }
    return ' <span class="propertyComment, border" data-container="body" data-toggle="popover" ' +
        'data-trigger="hover" popover data-placement="right" id="' + getNextEnumId() + '" data-content="' +
        escapeHtml(comment) + '"><span>...</span></span>';
}

String.prototype.stripTrailingSlash = function () {
    if (this.substr(-1) === '/') {
        return this.substr(0, this.length - 1);
    }
    return this;
};
String.prototype.ensureStartsWithSlash = function () {
    if (this.substr(0, 1) !== '/') {
        return '/' + this;
    }
    return this;
};
