export const defaultSdkConfig = (): any => ({
    build: {
        dropStyles: false
    },
    languages: {
        defaultLanguage: 'de',
        availableLanguages: ['de', 'fr', 'it', 'en'],
        showTranslationKeys: false
    },
    alert: {
        show: false,
        type: 'error',
        messages: [
            {key: 'error.authentication-failed'},
            {key: 'authentication.pages.messages.remaining-attempts.warning', arguments: {remainingAttempts: '2'}},
            {key: 'authentication.pages.messages.cancel'},
            {key: 'registration.pages.messages.completed'}
        ]
    },
    maintenanceMessage: {
        show: false,
        location: undefined,
        text: {
            de: 'Unser System wird in Kürze gewartet und wird in dieser Zeit nicht erreichbar sein.',
            en: 'Our system will be serviced shortly and will not be available during this time.',
            fr: 'Notre système sera révisé sous peu et ne sera pas disponible pendant cette période.',
            it: 'Il nostro sistema verrà sottoposto a manutenzione a breve e non sarà disponibile durante questo periodo.'
        }
    },
    buttons: {
        cancel: {
            show: true
        },
        goto: {
            show: false,
            targetStepIds: ['step1', 'step2']
        },
        clickable: []
    },
    checkboxes: {
        dynamicStepActivation: {
            show: false,
            targetStepIds: ['step1', 'step2']
        }
    },
    flows: {
        defaultFlowId: 'default',
        authentication: {
            latestAuthenticationFeedback: {
                show: false
            }
        }
    },
    // LOGINAPP-REST-UI-SDK-CONFIG-PAGE
    pages: [
        {
            path: '/auth/flow/{{FLOW}}/airlock-2fa/challenge',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/airlock-2fa/challenge-only',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/cronto/challenge',
            additionalAttributes: {
                'challenge': {
                    'onlineValidation': true,
                    'pushed': true,
                    'pushDevice': {
                        'label': 'My iPhone',
                        // the platform property is only used if label is set to null
                        'platform': 'IOS'
                    }
                }
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/password',
            pageSettings: {
                publicSelfServiceLink: '{{SPA}}/self-service/flow/default',
                userSelfRegistrationLink: '{{SPA}}/registration/flow/default'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/password/change',
            pageSettings: {
                oldPasswordRequired: false
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/mtan',
            additionalAttributes: {
                resendPossible: true,
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/mtan/registration/verification/otp',
            additionalAttributes: {
                resendPossible: true,
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/airlock-2fa/additional-activation',
            pageSettings: {
                showAppDeviceActivationLink: true
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/airlock-2fa/migration',
            pageSettings: {
                showAppDeviceActivationLink: true
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/message',
            additionalAttributes: {
                messageId: 'default'
            }
        },
        {
            path: 'auth/flow/{{FLOW}}/matrix',
            additionalAttributes: {
                matrixChallenges: ['A13']
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/oauth2/consent',
            additionalAttributes: {
                grantableScopes: [
                    {
                        scope: 'email',
                        granted: true,
                        updatedAt: '2024-10-23'
                    },
                    {
                        scope: 'phone',
                        granted: false,
                        updatedAt: '2023-08-15'
                    },
                    {
                        scope: 'address',
                        granted: false
                    }
                ]
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/email/otp',
            additionalAttributes: {
                resendPossible: true,
                emailAddress: 'john.doe@example.com'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/email/verification/otp/check',
            additionalAttributes: {
                emailAddress: 'john.doe@example.com'
            }
        },
        {
            path: '/auth/flow/{{FLOW}}/username',
            additionalAttributes: {
                captcha: {
                    // supported types are 'RE_CAPTCHA' and 'H_CAPTCHA' - the property 'key' can also be set to use your own site key
                    type: 'RE_CAPTCHA'
                }
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/challenge',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/challenge-only',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/approval/cronto/challenge',
            additionalAttributes: {
                'challenge': {
                    'onlineValidation': true,
                    'pushed': true,
                    'pushDevice': {
                        'label': 'My iPhone',
                        // the platform property is only used if label is set to null
                        'platform': 'IOS'
                    }
                }
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/approval/matrix',
            additionalAttributes: {
                matrixChallenges: ['A13']
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/approval/mtan',
            additionalAttributes: {
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/message',
            additionalAttributes: {
                messageId: 'default'
            }
        },
        {
            path: '/self-service/flow/{{FLOW}}/username',
            additionalAttributes: {
                captcha: {
                    type: 'RE_CAPTCHA'
                }
            }
        },
        {
            path: '/registration/flow/{{FLOW}}/data/example',
            additionalAttributes: {
                captcha: {
                    type: 'H_CAPTCHA'
                }
            }
        },
        {
            path: '/registration/flow/{{FLOW}}/message',
            additionalAttributes: {
                messageId: 'default'
            }
        },
        {
            path: '/registration/flow/{{FLOW}}/airlock-2fa/device/add',
            pageSettings: {
                showAppDeviceActivationLink: true
            }
        },
        {
            path: '/registration/flow/{{FLOW}}/verification/email',
            additionalAttributes: {
                captcha: null,
                emailAddress: 'john.doe@example.com'
            }
        },
        {
            path: '/registration/flow/{{FLOW}}/verification/phone-number',
            additionalAttributes: {
                captcha: null,
                resendPossible: true,
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/airlock-2fa/device/add',
            pageSettings: {
                showAppDeviceActivationLink: true
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/challenge',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/challenge-only',
            pageSettings: {
                showAppApprovalLink: true,
                // supported factors are 'ONLINE_QR_CODE' and 'ONE_TOUCH'
                factor: 'ONLINE_QR_CODE'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/approval/cronto/challenge',
            additionalAttributes: {
                'challenge': {
                    'onlineValidation': true,
                    'pushed': false,
                    'pushDevice': {
                        'label': 'My iPhone',
                        // the platform property is only used if label is set to null
                        'platform': 'IOS'
                    }
                }
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/approval/matrix',
            additionalAttributes: {
                matrixChallenges: ['A13']
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/approval/mtan',
            additionalAttributes: {
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/email/verification/otp/check',
            additionalAttributes: {
                emailAddress: 'john.doe@example.com'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/message',
            additionalAttributes: {
                messageId: 'default'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/mtan/registration/verification/otp',
            additionalAttributes: {
                resendPossible: true,
                phoneNumber: '+41791234567'
            }
        },
        {
            path: '/protected/flow/{{FLOW}}/vasco/device/activation',
            additionalAttributes: {
                activatableDevices: [
                    {
                        id: '0',
                        serialNumber: '000-1111-33'
                    },
                    {
                        id: '1',
                        serialNumber: '111-2222-44'
                    }
                ]
            }
        },
        {
            path: '/protected/portal',
            pageSettings: {
                portalGroups: [
                    {
                        identifier: 'self-services',
                        portalTargets: [
                            {identifier: 'cronto-device-management'},
                            {identifier: 'address-change'},
                            {identifier: 'password-change'}
                        ]
                    },
                    {
                        identifier: 'custom-group',
                        portalTargets: [
                            {identifier: 'custom-target'}
                        ]
                    }
                ]
            }
        },
        {
            path: '/error/message',
            pageSettings: {
                titleKey: 'general.error.title',
                message: {key: 'general.error.description', arguments: null}
            }
        }
    ],
    // LOGINAPP-REST-UI-SDK-CONFIG-PAGE
    // LOGINAPP-REST-UI-SDK-CONFIG-NAVIGATION
    navigation: {
        authentication: {
            password: [
                {path: '/auth/flow/{{FLOW}}/password'},
                {path: '/auth/flow/{{FLOW}}/password-only'},
                {path: '/auth/flow/{{FLOW}}/password/change'}
            ],
            airlock2fa: [
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/additional-activation'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/challenge'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/challenge-only'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/device/edit'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/device/selection'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/migration'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/passcode'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/qrcode-only'},
                {path: '/auth/flow/{{FLOW}}/airlock-2fa/usernameless'}
            ],
            mtan: [
                {path: '/auth/flow/{{FLOW}}/mtan'},
                {path: '/auth/flow/{{FLOW}}/mtan/registration/verification/iak'},
                {path: '/auth/flow/{{FLOW}}/mtan/registration/verification/otp'},
                {path: '/auth/flow/{{FLOW}}/mtan/selection'}
            ],
            fido: [
                {path: '/auth/flow/{{FLOW}}/fido'},
                {path: '/auth/flow/{{FLOW}}/fido/passwordless'},
                {path: '/auth/flow/{{FLOW}}/fido/registration'}
            ],
            cronto: [
                {path: '/auth/flow/{{FLOW}}/cronto/activation'},
                {path: '/auth/flow/{{FLOW}}/cronto/activation/complete'},
                {path: '/auth/flow/{{FLOW}}/cronto/activation/on-screen'},
                {path: '/auth/flow/{{FLOW}}/cronto/challenge'},
                {path: '/auth/flow/{{FLOW}}/cronto/push-activation'},
                {path: '/auth/flow/{{FLOW}}/cronto/selection'}
            ],
            email: [
                {path: '/auth/flow/{{FLOW}}/email/otp'},
                {path: '/auth/flow/{{FLOW}}/email/verification/otp/check'}
            ],
            otp: [
                {path: '/auth/flow/{{FLOW}}/oath-otp'},
                {path: '/auth/flow/{{FLOW}}/otp'},
                {path: '/auth/flow/{{FLOW}}/vasco-otp'}
            ],
            oauth2: [
                {path: '/auth/flow/{{FLOW}}/oauth2/consent'}
            ],
            secretQuestions: [
                {path: '/auth/flow/{{FLOW}}/secret-questions'}
            ],
            ssi: [
                {path: '/auth/flow/{{FLOW}}/ssi/invitation', incubating: true},
                {path: '/auth/flow/{{FLOW}}/ssi/issuance', incubating: true},
                {path: '/auth/flow/{{FLOW}}/ssi/verification', incubating: true}
            ],
            username: [
                {path: '/auth/flow/{{FLOW}}/username'}
            ],
            varia: [
                {path: '/auth/flow/{{FLOW}}/data/example'},
                {path: '/auth/flow/{{FLOW}}/matrix'},
                {path: '/auth/flow/{{FLOW}}/message'},
                {path: '/auth/flow/{{FLOW}}/migration-selection'},
                {path: '/auth/flow/{{FLOW}}/selection'},
                {path: '/auth/flow/{{FLOW}}/terms-of-service'},
                {path: '/auth/logout/disclaimer'}
            ],
            customUi: [
                {path: '/auth/flow/{{FLOW}}/ext/example'}
            ]
        },
        registration: {
            selfRegistration: [
                {path: '/registration/flow/{{FLOW}}/confirmation'},
                {path: '/registration/flow/{{FLOW}}/data/example'},
                {path: '/registration/flow/{{FLOW}}/message'},
                {path: '/registration/flow/{{FLOW}}/selection'},
                {path: '/registration/flow/{{FLOW}}/ssi/invitation', incubating: true},
                {path: '/registration/flow/{{FLOW}}/ssi/verification', incubating: true},
                {path: '/registration/flow/{{FLOW}}/terms-of-service'},
                {path: '/registration/flow/{{FLOW}}/verification/email'},
                {path: '/registration/flow/{{FLOW}}/verification/phone-number'}
            ],
            tokens: [
                {path: '/registration/flow/{{FLOW}}/airlock-2fa/device/add'}
            ],
            customUi: [
                {path: '/registration/flow/{{FLOW}}/ext/example'}
            ]
        },
        publicSelfServices: {
            selfServices: [
                {path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/challenge'},
                {path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/challenge-only'},
                {path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/qrcode'},
                {path: '/self-service/flow/{{FLOW}}/approval/airlock-2fa/selection'},
                {path: '/self-service/flow/{{FLOW}}/approval/cronto/challenge'},
                {path: '/self-service/flow/{{FLOW}}/approval/cronto/selection'},
                {path: '/self-service/flow/{{FLOW}}/approval/fido'},
                {path: '/self-service/flow/{{FLOW}}/approval/matrix'},
                {path: '/self-service/flow/{{FLOW}}/approval/mtan'},
                {path: '/self-service/flow/{{FLOW}}/approval/mtan/selection'},
                {path: '/self-service/flow/{{FLOW}}/approval/vasco-otp'},
                {path: '/self-service/flow/{{FLOW}}/confirmation'},
                {path: '/self-service/flow/{{FLOW}}/email'},
                {path: '/self-service/flow/{{FLOW}}/message'},
                {path: '/self-service/flow/{{FLOW}}/mtan'},
                {path: '/self-service/flow/{{FLOW}}/password'},
                {path: '/self-service/flow/{{FLOW}}/secret-questions'},
                {path: '/self-service/flow/{{FLOW}}/selection'},
                {path: '/self-service/flow/{{FLOW}}/ssi/invitation', incubating: true},
                {path: '/self-service/flow/{{FLOW}}/ssi/verification', incubating: true},
                {path: '/self-service/flow/{{FLOW}}/username'}
            ],
            customUi: [
                {path: '/self-service/flow/{{FLOW}}/ext/example'}
            ]
        },
        protectedSelfServices: {
            selfServices: [
                {path: '/protected/account-links'},
                {path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/challenge'},
                {path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/challenge-only'},
                {path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/qrcode'},
                {path: '/protected/flow/{{FLOW}}/approval/airlock-2fa/selection'},
                {path: '/protected/flow/{{FLOW}}/approval/cronto/challenge'},
                {path: '/protected/flow/{{FLOW}}/approval/cronto/selection'},
                {path: '/protected/flow/{{FLOW}}/approval/fido'},
                {path: '/protected/flow/{{FLOW}}/approval/matrix'},
                {path: '/protected/flow/{{FLOW}}/approval/mtan'},
                {path: '/protected/flow/{{FLOW}}/approval/mtan/selection'},
                {path: '/protected/flow/{{FLOW}}/approval/vasco-otp'},
                {path: '/protected/flow/{{FLOW}}/confirmation'},
                {path: '/protected/flow/{{FLOW}}/data/example'},
                {path: '/protected/flow/{{FLOW}}/email/verification/otp/check'},
                {path: '/protected/flow/{{FLOW}}/message'},
                {path: '/protected/flow/{{FLOW}}/mtan/registration/verification/iak'},
                {path: '/protected/flow/{{FLOW}}/mtan/registration/verification/otp'},
                {path: '/protected/flow/{{FLOW}}/password/change'},
                {path: '/protected/flow/{{FLOW}}/selection'},
                {path: '/protected/flow/{{FLOW}}/ssi/invitation', incubating: true},
                {path: '/protected/flow/{{FLOW}}/ssi/issuance', incubating: true},
                {path: '/protected/flow/{{FLOW}}/vasco/device/activation'},
                {path: '/protected/oauth2/sessions'},
                {path: '/protected/portal'},
                {path: '/protected/remember-me/devices'}
            ],
            tokens: [
                {path: '/protected/flow/{{FLOW}}/airlock-2fa/device/add'},
                {path: '/protected/flow/{{FLOW}}/airlock-2fa/device/edit'},
                {path: '/protected/flow/{{FLOW}}/cronto/activation'},
                {path: '/protected/flow/{{FLOW}}/cronto/activation/complete'},
                {path: '/protected/flow/{{FLOW}}/cronto/activation/on-screen'},
                {path: '/protected/flow/{{FLOW}}/cronto/device/edit'},
                {path: '/protected/flow/{{FLOW}}/cronto/push-activation'},
                {path: '/protected/flow/{{FLOW}}/fido/credential/edit'},
                {path: '/protected/flow/{{FLOW}}/fido/registration'},
                {path: '/protected/tokens/airlock-2fa/devices'},
                {path: '/protected/tokens/cronto/devices'},
                {path: '/protected/tokens/fido/credentials'},
                {path: '/protected/tokens/mtan'}
            ],
            customUi: [
                {path: '/protected/flow/{{FLOW}}/ext/example'}
            ]
        },
        error: {
            error: [
                {path: '/error'},
                {path: '/error/message'},
                {path: '/error/no-access'},
                {path: '/error/not-available'},
                {path: '/error/not-found'}
            ]
        }
    },
    // LOGINAPP-REST-UI-SDK-CONFIG-NAVIGATION
    customResponses: []
});
