/*
 Migrates the Airlock IAM DB schema from version 7.3 to version 7.4.
 Written for H2 DB.
 */

-- Drop medusa_user constraints
ALTER TABLE medusa_user
    DROP PRIMARY KEY;

-- Add named medusa_user constraints
ALTER TABLE medusa_user
    ADD CONSTRAINT pk_medusa_user PRIMARY KEY (username);

-- --------------------

-- Drop medusa_admin constraints
ALTER TABLE medusa_admin
    DROP PRIMARY KEY;

-- Add named medusa_admin constraints
ALTER TABLE medusa_admin
    ADD CONSTRAINT pk_medusa_admin PRIMARY KEY (username);

-- --------------------

-- Drop medusa_maint_msg constraints
ALTER TABLE medusa_maint_msg
    DROP PRIMARY KEY;

-- Add named medusa_maint_msg constraints
ALTER TABLE medusa_maint_msg
    ADD CONSTRAINT pk_medusa_maint_msg PRIMARY KEY (name);

-- --------------------

-- Drop medusa_maint_msg_tnsl constraints
ALTER TABLE medusa_maint_msg_tnsl
    DROP PRIMARY KEY;

-- Add named medusa_maint_msg_tnsl constraints
ALTER TABLE medusa_maint_msg_tnsl
    ADD CONSTRAINT pk_medusa_maint_msg_tnsl PRIMARY KEY (message_ref, language);

-- --------------------

-- Drop token_assignment constraints
ALTER TABLE token_assignment
    DROP PRIMARY KEY;
ALTER TABLE token_assignment
    DROP CONSTRAINT fk_ta_token_id;

-- Drop token constraints
ALTER TABLE token
    DROP CONSTRAINT fk_obsoletes_token_id;
ALTER TABLE token
    DROP CONSTRAINT fk_activates_token_id;
ALTER TABLE token
    DROP PRIMARY KEY;

-- Add named token constraints
ALTER TABLE token
    ADD CONSTRAINT pk_token_id PRIMARY KEY (token_id);
CREATE INDEX IF NOT EXISTS obsoletes_token_id_idx
    ON token (obsoletes_token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_obsoletes_token_id FOREIGN KEY (obsoletes_token_id) REFERENCES token (token_id);
CREATE INDEX IF NOT EXISTS activates_token_id_index
    ON token (activates_token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_activates_token_id FOREIGN KEY (activates_token_id) REFERENCES token (token_id);

-- Add named token_assignment constraints
ALTER TABLE token_assignment
    ADD CONSTRAINT pk_ta_token_id_user PRIMARY KEY (ta_token_id, ta_user);
ALTER TABLE token_assignment
    ADD CONSTRAINT fk_ta_token_id FOREIGN KEY (ta_token_id) REFERENCES token (token_id);

-- --------------------

-- Drop medusa_pwd_selfservice_token constraints
ALTER TABLE medusa_pwd_selfservice_token
    DROP PRIMARY KEY;

-- Add named medusa_pwd_selfservice_token constraints
ALTER TABLE medusa_pwd_selfservice_token
    ADD CONSTRAINT pk_pwd_selfservice_token PRIMARY KEY (token_id);

-- --------------------

-- Drop login_history constraints
ALTER TABLE login_history
    DROP PRIMARY KEY;

-- Add named login_history constraints
ALTER TABLE login_history
    ADD CONSTRAINT pk_login_history PRIMARY KEY (id);

-- --------------------

-- Drop account_link constraints
ALTER TABLE account_link
    DROP PRIMARY KEY;

-- Add named account_link constraints
ALTER TABLE account_link
    ADD CONSTRAINT pk_account_link PRIMARY KEY (link_id);

-- --------------------

-- Drop user_consent constraints
ALTER TABLE user_consent
    DROP PRIMARY KEY;

-- Add named user_consent constraints
ALTER TABLE user_consent
    ADD CONSTRAINT pk_user_consent PRIMARY KEY (consent_id);

-- --------------------

-- Drop oauth2_attribute_translation constraints
ALTER TABLE oauth2_attribute_translation
    DROP CONSTRAINT fk_oauth2_attributes_pk;

-- Drop oauth2_attributes constraints
ALTER TABLE oauth2_attributes
    DROP PRIMARY KEY;
ALTER TABLE oauth2_attributes
    DROP CONSTRAINT fk_oa_principal_pk;

-- Drop api_key constraints
ALTER TABLE api_key
    DROP CONSTRAINT fk_api_key_principal_pk;

-- Drop client_cert_credential constraints
ALTER TABLE client_cert_credential
    DROP CONSTRAINT fk_ccc_principal_pk;

-- Drop principal constraints
ALTER TABLE principal
    DROP PRIMARY KEY;
ALTER TABLE principal
    DROP CONSTRAINT fk_subscription_pk;

-- Drop plan_usage constraints
ALTER TABLE plan_usage
    DROP CONSTRAINT fk_plan_usage_subscription_pk;

-- Drop subscription constraints
ALTER TABLE subscription
    DROP PRIMARY KEY;

-- Add named subscription constraints
ALTER TABLE subscription
    ADD CONSTRAINT pk_subscription PRIMARY KEY (pk);

-- Add named plan_usage constraints
CREATE INDEX IF NOT EXISTS fk_plan_usage_subscription_pk
    ON plan_usage (subscription_pk);
ALTER TABLE plan_usage
    ADD CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE;

-- Add named principal constraints
ALTER TABLE principal
    ADD CONSTRAINT pk_principal PRIMARY KEY (pk);
CREATE INDEX IF NOT EXISTS subscription_idx
    ON principal (subscription_pk);
ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);

-- Add named client_cert_credential constraints
CREATE INDEX IF NOT EXISTS principal_pk_idx
    ON client_cert_credential (principal_pk);
ALTER TABLE client_cert_credential
    ADD CONSTRAINT fk_ccc_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named api_key constraints
CREATE INDEX IF NOT EXISTS fk_api_key_principal_pk
    ON api_key (principal_pk);
ALTER TABLE api_key
    ADD CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attributes constraints
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT pk_oauth2_attributes PRIMARY KEY (pk);
CREATE UNIQUE INDEX IF NOT EXISTS oa2_attrs_principal_idx_unique
    ON oauth2_attributes (principal_pk);
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT fk_oa_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attribute_translation constraints
CREATE INDEX IF NOT EXISTS oauth2_attributes_pk_idx
    ON oauth2_attribute_translation (oauth2_attributes_pk);
ALTER TABLE oauth2_attribute_translation
    ADD CONSTRAINT fk_oauth2_attributes_pk FOREIGN KEY (oauth2_attributes_pk) REFERENCES oauth2_attributes (pk) ON DELETE CASCADE;

-- --------------------

-- Drop oauth2_session_attribute constraints
ALTER TABLE oauth2_session_attribute
    DROP CONSTRAINT fk_oauth2_session_pk;

-- Drop oauth2_session constraints
ALTER TABLE oauth2_session
    DROP PRIMARY KEY;

-- Add named oauth2_session constraints
ALTER TABLE oauth2_session
    ADD CONSTRAINT pk_oauth2_session PRIMARY KEY (pk);

-- Add named oauth2_session_attribute constraints
CREATE INDEX IF NOT EXISTS ix_osa_oauth2_session_pk
    ON oauth2_session_attribute (oauth2_session_pk);
ALTER TABLE oauth2_session_attribute
    ADD CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE;

-- --------------------

-- Drop airlock_2fa_activation_letter constraints
ALTER TABLE airlock_2fa_activation_letter
    DROP PRIMARY KEY;
ALTER TABLE airlock_2fa_activation_letter
    DROP CONSTRAINT fk_activation_letter_user_pk;

-- Drop airlock_2fa_user constraints
ALTER TABLE airlock_2fa_user
    DROP PRIMARY KEY;

-- Add named airlock_2fa_user constraints
ALTER TABLE airlock_2fa_user
    ADD CONSTRAINT pk_airlock_2fa_user PRIMARY KEY (pk);

-- Add named airlock_2fa_activation_letter constraints
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT pk_2fa_activation_letter PRIMARY KEY (pk);
CREATE INDEX IF NOT EXISTS fk_activation_letter_user_pk
    ON airlock_2fa_activation_letter (airlock_2fa_user_pk);
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE;

-- --------------------

-- Drop last_selection constraints
ALTER TABLE last_selection
    DROP PRIMARY KEY;

-- Add named last_selection constraints
ALTER TABLE last_selection
    ADD CONSTRAINT pk_last_selection PRIMARY KEY (last_selection_id);

-- --------------------

ALTER TABLE token
    ALTER COLUMN token_id BIGINT NOT NULL AUTO_INCREMENT;
ALTER TABLE token
    ALTER COLUMN obsoletes_token_id BIGINT;
ALTER TABLE token
    ALTER COLUMN activates_token_id BIGINT;
ALTER TABLE token_assignment
    ALTER COLUMN ta_token_id BIGINT NOT NULL;

-- --------------------

ALTER TABLE medusa_user
    ADD realm VARCHAR(50) AFTER remember_me_gen_date;
ALTER TABLE medusa_admin
    ADD realm VARCHAR(50) AFTER email;

-- --------------------

CREATE TABLE fido_user
(
    pk               BIGINT       NOT NULL AUTO_INCREMENT,
    iam_user_id      VARCHAR(100) NOT NULL,
    user_handle      VARCHAR(36)  NOT NULL,
    relying_party_id VARCHAR(255) NOT NULL,
    tenant_id        VARCHAR(50)  NOT NULL,

    CONSTRAINT pk_fido_user PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_user_handle
    ON fido_user (user_handle);
CREATE UNIQUE INDEX ux_fido_iam_user_rpid
    ON fido_user (iam_user_id, relying_party_id, tenant_id);

CREATE TABLE fido_credential
(
    id                            VARCHAR(36)       NOT NULL,
    fido_user_pk                  BIGINT            NOT NULL,
    name                          VARCHAR(100)      NOT NULL,
    display_name                  VARCHAR(100)      NOT NULL,
    public_key_credential_id      BLOB              NOT NULL,
    public_key_credential_id_hash VARBINARY(64)     NOT NULL,
    public_key                    VARBINARY(1024)   NOT NULL,
    signature_counter             BIGINT  DEFAULT 0 NOT NULL,
    resident_requested            TINYINT           NOT NULL,
    attestation_object            BLOB              NOT NULL,
    registered_at                 BIGINT            NOT NULL,
    first_used_at                 BIGINT,
    last_used_at                  BIGINT,
    total_usages                  INTEGER DEFAULT 0 NOT NULL,
    locked                        TINYINT           NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_fido_credential_user_pk
    ON fido_credential (fido_user_pk);
ALTER TABLE fido_credential
    ADD CONSTRAINT fk_fido_credential_user_pk FOREIGN KEY (fido_user_pk) REFERENCES fido_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_cred_id
    ON fido_credential (id);
CREATE UNIQUE INDEX ux_fido_cred_id_hash
    ON fido_credential (public_key_credential_id_hash);
