/*
 Migrates the Airlock IAM DB schema from version 7.1 to version 7.2.
 Written for MariaDB.
 */

CREATE TABLE IF NOT EXISTS subscription
(
    pk           INTEGER                               NOT NULL AUTO_INCREMENT,
    id           VARCHAR(36) COLLATE latin1_nopad_bin  NOT NULL,
    display_name VARCHAR(100)                          NOT NULL,
    tenant_id    VARCHAR(50) COLLATE utf8mb4_nopad_bin NOT NULL,

    PRIMARY KEY (pk)
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
CREATE UNIQUE INDEX subscription_id_idx_unique
    ON subscription (id);

ALTER TABLE principal
    ADD COLUMN description VARCHAR(100);
ALTER TABLE principal
    ADD COLUMN label VARCHAR(100);
ALTER TABLE principal
    ADD COLUMN subscription_pk INTEGER;

ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);
CREATE INDEX subscription_idx
    ON principal (subscription_pk);

ALTER TABLE oauth2_attributes
    MODIFY COLUMN client_secret VARCHAR(450);

CREATE TABLE airlock_2fa_user
(
    pk                  INTEGER                                NOT NULL AUTO_INCREMENT,
    iam_user_id         VARCHAR(100) COLLATE utf8mb4_nopad_bin NOT NULL,
    airlock_2fa_user_id VARCHAR(50) COLLATE latin1_nopad_bin   NOT NULL,
    tenant_id           VARCHAR(50) COLLATE utf8mb4_nopad_bin  NOT NULL,

    PRIMARY KEY (pk)
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
CREATE UNIQUE INDEX a2fa_iam_user_id_idx_unique
    ON airlock_2fa_user (iam_user_id, tenant_id);
CREATE UNIQUE INDEX a2fa_user_id_idx_unique
    ON airlock_2fa_user (airlock_2fa_user_id);

CREATE TABLE airlock_2fa_activation_letter
(
    pk                  INTEGER                                NOT NULL AUTO_INCREMENT,
    id                  VARCHAR(36) COLLATE latin1_nopad_bin   NOT NULL,
    activation_code     VARCHAR(750) COLLATE latin1_nopad_bin  NOT NULL,
    airlock_2fa_user_pk INTEGER                                NOT NULL,
    creator             VARCHAR(100) COLLATE utf8mb4_nopad_bin NOT NULL,

    PRIMARY KEY (pk),
    CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;

CREATE UNIQUE INDEX a2fa_letter_id_idx_unique
    ON airlock_2fa_activation_letter (id);

CREATE TABLE api_key
(
    id            VARCHAR(36) COLLATE latin1_nopad_bin  NOT NULL,
    principal_pk  INTEGER                               NOT NULL,
    fingerprint   VARCHAR(30) COLLATE latin1_nopad_bin  NOT NULL,
    secret        VARCHAR(450) COLLATE latin1_nopad_bin NOT NULL,
    description   VARCHAR(100),
    valid_from    DATETIME(6),
    valid_to      DATETIME(6),
    locked        INTEGER DEFAULT 0                     NOT NULL,
    creation_date DATETIME(6)                           NOT NULL,
    tenant_id     VARCHAR(50) COLLATE utf8mb4_nopad_bin NOT NULL,

    CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
CREATE UNIQUE INDEX api_key_id_idx_unique
    ON api_key (id);

CREATE UNIQUE INDEX api_key_fp_idx_unique
    ON api_key (fingerprint, tenant_id);

CREATE TABLE IF NOT EXISTS plan_usage
(
    id              VARCHAR(36) COLLATE latin1_nopad_bin NOT NULL,
    subscription_pk INTEGER                              NOT NULL,
    name            VARCHAR(100)                         NOT NULL,
    rate_limit      INTEGER,

    CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
CREATE UNIQUE INDEX plan_usage_subscr_idx_unique
    ON plan_usage (name, subscription_pk);
CREATE UNIQUE INDEX plan_usage_id_idx_unique
    ON plan_usage (id);

