/*
 Migrates the Airlock IAM DB schema from version 7.3 to version 7.4.
 Written for MariaDB.
 */

ALTER TABLE medusa_user
    ADD realm VARCHAR(50) AFTER remember_me_gen_date;
ALTER TABLE medusa_admin
    ADD realm VARCHAR(50) AFTER email;

-- --------------------

CREATE TABLE fido_user
(
    pk               BIGINT                                        NOT NULL AUTO_INCREMENT,
    iam_user_id      VARCHAR(100) COLLATE utf8mb4_general_nopad_ci NOT NULL,
    user_handle      VARCHAR(36) COLLATE utf8mb4_nopad_bin         NOT NULL,
    relying_party_id VARCHAR(255) COLLATE utf8mb4_general_nopad_ci NOT NULL,
    tenant_id        VARCHAR(50) COLLATE utf8mb4_nopad_bin         NOT NULL,

    CONSTRAINT pk_fido_user PRIMARY KEY (pk)
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_user_handle
    ON fido_user (user_handle);
CREATE UNIQUE INDEX ux_fido_iam_user_rpid
    ON fido_user (iam_user_id, relying_party_id, tenant_id);

CREATE TABLE fido_credential
(
    id                            VARCHAR(36) COLLATE utf8mb4_nopad_bin         NOT NULL,
    fido_user_pk                  BIGINT                                        NOT NULL,
    name                          VARCHAR(100) COLLATE utf8mb4_general_nopad_ci NOT NULL,
    display_name                  VARCHAR(100) COLLATE utf8mb4_general_nopad_ci NOT NULL,
    public_key_credential_id      BLOB                                          NOT NULL,
    public_key_credential_id_hash VARBINARY(64)                                 NOT NULL,
    public_key                    VARBINARY(1024)                               NOT NULL,
    signature_counter             BIGINT  DEFAULT 0                             NOT NULL,
    resident_requested            TINYINT                                       NOT NULL,
    attestation_object            BLOB                                          NOT NULL,
    registered_at                 BIGINT                                        NOT NULL,
    first_used_at                 BIGINT,
    last_used_at                  BIGINT,
    total_usages                  INTEGER DEFAULT 0                             NOT NULL,
    locked                        TINYINT                                       NOT NULL
) CHARACTER SET utf8mb4
  COLLATE utf8mb4_unicode_520_nopad_ci;
-- FOREIGN KEY indexes
CREATE INDEX ix_fido_credential_user_pk
    ON fido_credential (fido_user_pk);
ALTER TABLE fido_credential
    ADD CONSTRAINT fk_fido_credential_user_pk FOREIGN KEY (fido_user_pk) REFERENCES fido_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_cred_id
    ON fido_credential (id);
CREATE UNIQUE INDEX ux_fido_cred_id_hash
    ON fido_credential (public_key_credential_id_hash);
