/*
 Migrates the Airlock IAM DB schema from version 7.1 to version 7.2.
 Written for MSSQL.
 */

CREATE TABLE subscription
(
    pk           INT                                                NOT NULL IDENTITY (1, 1),
    id           NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    display_name NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    tenant_id    NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,

    PRIMARY KEY (pk)
);
CREATE UNIQUE INDEX subscription_id_idx_unique
    ON subscription (id);

ALTER TABLE principal
    ADD description NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS;
ALTER TABLE principal
    ADD label NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS;
ALTER TABLE principal
    ADD subscription_pk INT;

ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);
CREATE INDEX subscription_idx
    ON principal (subscription_pk);

ALTER TABLE oauth2_attributes
    ALTER COLUMN client_secret NVARCHAR(450) COLLATE SQL_Latin1_General_CP1_CS_AS;

CREATE TABLE airlock_2fa_user
(
    pk                  INT                                                NOT NULL IDENTITY (1, 1),
    iam_user_id         NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    airlock_2fa_user_id NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    tenant_id           NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,

    PRIMARY KEY (pk)
);
CREATE UNIQUE INDEX a2fa_iam_user_id_idx_unique
    ON airlock_2fa_user (iam_user_id, tenant_id);
CREATE UNIQUE INDEX a2fa_user_id_idx_unique
    ON airlock_2fa_user (airlock_2fa_user_id);

CREATE TABLE airlock_2fa_activation_letter
(
    pk                  INT                                                NOT NULL IDENTITY (1, 1),
    id                  NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    activation_code     NVARCHAR(750) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    airlock_2fa_user_pk INT                                                NOT NULL,
    creator             NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,

    PRIMARY KEY (pk),
    CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE
);

CREATE UNIQUE INDEX a2fa_letter_id_idx_unique
    ON airlock_2fa_activation_letter (id);


CREATE TABLE api_key
(
    id            NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    principal_pk  INT                                                NOT NULL,
    fingerprint   NVARCHAR(30) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    secret        NVARCHAR(450) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    description   NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS,
    valid_from    DATETIME2,
    valid_to      DATETIME2,
    locked        INT DEFAULT 0                                      NOT NULL,
    creation_date DATETIME2                                          NOT NULL,
    tenant_id     NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,

    CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE
);
CREATE UNIQUE INDEX api_key_id_idx_unique
    ON api_key (id);

CREATE UNIQUE INDEX api_key_fp_idx_unique
    ON api_key (fingerprint, tenant_id);

CREATE TABLE plan_usage
(
    id              NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    subscription_pk INT                                                NOT NULL,
    name            NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    rate_limit      INT,

    CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE
);
CREATE UNIQUE INDEX plan_usage_subscr_idx_unique
    ON plan_usage (name, subscription_pk);
CREATE UNIQUE INDEX plan_usage_id_idx_unique
    ON plan_usage (id);

