ALTER TABLE token_assignment
    DROP CONSTRAINT fk_ta_token_id;
ALTER TABLE token
    DROP CONSTRAINT fk_activates_token_id;
ALTER TABLE token
    DROP CONSTRAINT fk_obsoletes_token_id;

DROP INDEX obsoletes_token_id_idx ON token;
DROP INDEX activates_token_id_index ON token;
DROP INDEX ta_index ON token_assignment;

DECLARE @ConstraintName nvarchar(200);
-- drop PK
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
where TABLE_NAME = 'token'
  AND COLUMN_NAME = 'token_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE token DROP CONSTRAINT ' + @ConstraintName);
alter table token
    alter column token_id bigint not null;

-- drop constraint
SELECT @ConstraintName = Name
FROM SYS.DEFAULT_CONSTRAINTS
WHERE PARENT_OBJECT_ID = OBJECT_ID('token')
  AND PARENT_COLUMN_ID = (SELECT column_id FROM sys.columns WHERE NAME = N'active' AND object_id = OBJECT_ID(N'token'));
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE token DROP CONSTRAINT ' + @ConstraintName);
alter table token
    alter column active TINYINT NOT NULL;
-- add 'default 0' constraint
ALTER TABLE token
    ADD DEFAULT 0 FOR active;


alter table token
    alter column obsoletes_token_id bigint null;
alter table token
    alter column activates_token_id bigint null;

-- drop FK
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
where TABLE_NAME = 'token_assignment'
  AND COLUMN_NAME = 'ta_token_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE token_assignment DROP CONSTRAINT ' + @ConstraintName);
alter table token_assignment
    alter column ta_token_id bigint not null;

-- drop constraint
SELECT @ConstraintName = Name
FROM SYS.DEFAULT_CONSTRAINTS
WHERE PARENT_OBJECT_ID = OBJECT_ID('token_assignment')
  AND PARENT_COLUMN_ID =
      (SELECT column_id FROM sys.columns WHERE NAME = N'ta_order_new' AND object_id = OBJECT_ID(N'token_assignment'));
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE token_assignment DROP CONSTRAINT ' + @ConstraintName);
alter table token_assignment
    alter column ta_order_new tinyint not null;
-- add 'default 0' constraint
ALTER TABLE token_assignment
    ADD DEFAULT 0 FOR ta_order_new;

ALTER TABLE token
    ADD CONSTRAINT pk_token_id PRIMARY KEY (token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_obsoletes_token_id FOREIGN KEY (obsoletes_token_id) REFERENCES token (token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_activates_token_id FOREIGN KEY (activates_token_id) REFERENCES token (token_id);

ALTER TABLE token_assignment
    ADD CONSTRAINT pk_ta_token_id_user PRIMARY KEY (ta_token_id, ta_user);
ALTER TABLE token_assignment
    ADD CONSTRAINT fk_ta_token_id FOREIGN KEY (ta_token_id) REFERENCES token (token_id);

CREATE INDEX activates_token_id_index ON token (activates_token_id);
CREATE INDEX obsoletes_token_id_idx ON token (obsoletes_token_id);
CREATE INDEX ta_index ON token_assignment (ta_user, ta_token_id);


CREATE TABLE oauth2_session
(
    pk BIGINT                                            NOT NULL IDENTITY (1, 1),
    id NVARCHAR(40) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,

    PRIMARY KEY (pk)
);
CREATE UNIQUE INDEX ux_oauth2_session_id
    ON oauth2_session (id);

CREATE TABLE oauth2_session_attribute
(
    oauth2_session_pk BIGINT                                            NOT NULL,
    name              NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    value             NVARCHAR(200) COLLATE Latin1_General_CI_AI        NOT NULL,

    CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE
);
CREATE INDEX ix_osa_oauth2_session_pk
    ON oauth2_session_attribute (oauth2_session_pk);

CREATE TABLE last_selection
(
    last_selection_id INT                                                NOT NULL IDENTITY (1, 1),
    username          NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    option_id         NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    selected_at       DATETIME2                                          NOT NULL,
    step_id           NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    tenant_id         NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,

    PRIMARY KEY (last_selection_id)
);
CREATE UNIQUE INDEX ux_last_selection
    ON last_selection (username, tenant_id, step_id);
