/*
 Migrates the Airlock IAM DB schema from version 7.3 to version 7.4.
 Written for MSSQL.
 */

-- Drop medusa_user constraints
DECLARE @ConstraintName nvarchar(200);
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'medusa_user'
  AND COLUMN_NAME = 'username';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE medusa_user DROP CONSTRAINT ' + @ConstraintName);

-- Add named medusa_user constraints
ALTER TABLE medusa_user
    ADD CONSTRAINT pk_medusa_user PRIMARY KEY (username);

-- --------------------

-- Drop medusa_admin constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'medusa_admin'
  AND COLUMN_NAME = 'username';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE medusa_admin DROP CONSTRAINT ' + @ConstraintName);

-- Add named medusa_admin constraints
ALTER TABLE medusa_admin
    ADD CONSTRAINT pk_medusa_admin PRIMARY KEY (username);

-- --------------------

-- Drop medusa_maint_msg constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'medusa_maint_msg'
  AND COLUMN_NAME = 'name';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE medusa_maint_msg DROP CONSTRAINT ' + @ConstraintName);

-- Add named medusa_maint_msg constraints
ALTER TABLE medusa_maint_msg
    ADD CONSTRAINT pk_medusa_maint_msg PRIMARY KEY (name);

-- --------------------

-- Drop medusa_maint_msg_tnsl constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'medusa_maint_msg_tnsl'
  AND COLUMN_NAME = 'message_ref';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE medusa_maint_msg_tnsl DROP CONSTRAINT ' + @ConstraintName);

-- Add named medusa_maint_msg_tnsl constraints
ALTER TABLE medusa_maint_msg_tnsl
    ADD CONSTRAINT pk_medusa_maint_msg_tnsl PRIMARY KEY (message_ref, language);

-- --------------------

-- Drop medusa_pwd_selfservice_token constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'medusa_pwd_selfservice_token'
  AND COLUMN_NAME = 'token_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE medusa_pwd_selfservice_token DROP CONSTRAINT ' + @ConstraintName);

-- Add named medusa_pwd_selfservice_token constraints
ALTER TABLE medusa_pwd_selfservice_token
    ADD CONSTRAINT pk_pwd_selfservice_token PRIMARY KEY (token_id);

-- --------------------

-- Drop login_history constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'login_history'
  AND COLUMN_NAME = 'id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE login_history DROP CONSTRAINT ' + @ConstraintName);

-- Add named login_history constraints
ALTER TABLE login_history
    ADD CONSTRAINT pk_login_history PRIMARY KEY (id);

-- --------------------

-- Drop account_link constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'account_link'
  AND COLUMN_NAME = 'link_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE account_link DROP CONSTRAINT ' + @ConstraintName);

-- Add named account_link constraints
ALTER TABLE account_link
    ADD CONSTRAINT pk_account_link PRIMARY KEY (link_id);

-- --------------------

-- Drop user_consent constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'user_consent'
  AND COLUMN_NAME = 'consent_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE user_consent DROP CONSTRAINT ' + @ConstraintName);

-- Add named user_consent constraints
ALTER TABLE user_consent
    ADD CONSTRAINT pk_user_consent PRIMARY KEY (consent_id);

-- --------------------

-- Drop oauth2_attribute_translation constraints
ALTER TABLE oauth2_attribute_translation
    DROP CONSTRAINT fk_oauth2_attributes_pk;

-- Drop oauth2_attributes constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'oauth2_attributes'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE oauth2_attributes DROP CONSTRAINT ' + @ConstraintName);
ALTER TABLE oauth2_attributes
    DROP CONSTRAINT fk_oa_principal_pk;

-- Drop api_key constraints
ALTER TABLE api_key
    DROP CONSTRAINT fk_api_key_principal_pk;

-- Drop client_cert_credential constraints
ALTER TABLE client_cert_credential
    DROP CONSTRAINT fk_ccc_principal_pk;

-- Drop principal constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'principal'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE principal DROP CONSTRAINT ' + @ConstraintName);
ALTER TABLE principal
    DROP CONSTRAINT fk_subscription_pk;

-- Drop plan_usage constraints
ALTER TABLE plan_usage
    DROP CONSTRAINT fk_plan_usage_subscription_pk;

-- Drop subscription constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'subscription'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE subscription DROP CONSTRAINT ' + @ConstraintName);

-- Add named subscription constraints
ALTER TABLE subscription
    ADD CONSTRAINT pk_subscription PRIMARY KEY (pk);

-- Add named plan_usage constraints
IF NOT EXISTS(SELECT *
              FROM sys.indexes
              WHERE name = 'fk_plan_usage_subscription_pk'
                AND object_id = OBJECT_ID('plan_usage'))
    BEGIN
        CREATE INDEX fk_plan_usage_subscription_pk ON plan_usage (subscription_pk);
    END
ALTER TABLE plan_usage
    ADD CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE;

-- Add named principal constraints
ALTER TABLE principal
    ADD CONSTRAINT pk_principal PRIMARY KEY (pk);
ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);

-- Add named client_cert_credential constraints
ALTER TABLE client_cert_credential
    ADD CONSTRAINT fk_ccc_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named api_key constraints
IF NOT EXISTS(SELECT *
              FROM sys.indexes
              WHERE name = 'fk_api_key_principal_pk'
                AND object_id = OBJECT_ID('api_key'))
    BEGIN
        CREATE INDEX fk_api_key_principal_pk ON api_key (principal_pk);
    END
ALTER TABLE api_key
    ADD CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attributes constraints
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT pk_oauth2_attributes PRIMARY KEY (pk);
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT fk_oa_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attribute_translation constraints
ALTER TABLE oauth2_attribute_translation
    ADD CONSTRAINT fk_oauth2_attributes_pk FOREIGN KEY (oauth2_attributes_pk) REFERENCES oauth2_attributes (pk) ON DELETE CASCADE;

-- --------------------

-- Drop oauth2_session_attribute constraints
ALTER TABLE oauth2_session_attribute
    DROP CONSTRAINT fk_oauth2_session_pk;

-- Drop oauth2_session constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'oauth2_session'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE oauth2_session DROP CONSTRAINT ' + @ConstraintName);

-- Add named oauth2_session constraints
ALTER TABLE oauth2_session
    ADD CONSTRAINT pk_oauth2_session PRIMARY KEY (pk);

-- Add named oauth2_session_attribute constraints
ALTER TABLE oauth2_session_attribute
    ADD CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE;

-- --------------------

-- Drop airlock_2fa_activation_letter constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'airlock_2fa_activation_letter'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE airlock_2fa_activation_letter DROP CONSTRAINT ' + @ConstraintName);
ALTER TABLE airlock_2fa_activation_letter
    DROP CONSTRAINT fk_activation_letter_user_pk;

-- Drop airlock_2fa_user constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'airlock_2fa_user'
  AND COLUMN_NAME = 'pk';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE airlock_2fa_user DROP CONSTRAINT ' + @ConstraintName);

-- Add named airlock_2fa_user constraints
ALTER TABLE airlock_2fa_user
    ADD CONSTRAINT pk_airlock_2fa_user PRIMARY KEY (pk);

-- Add named airlock_2fa_activation_letter constraints
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT pk_2fa_activation_letter PRIMARY KEY (pk);
IF NOT EXISTS(SELECT *
              FROM sys.indexes
              WHERE name = 'fk_activation_letter_user_pk'
                AND object_id = OBJECT_ID('airlock_2fa_activation_letter'))
    BEGIN
        CREATE INDEX fk_activation_letter_user_pk ON airlock_2fa_activation_letter (airlock_2fa_user_pk);
    END
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE;

-- --------------------

-- Drop last_selection constraints
SELECT @ConstraintName = CONSTRAINT_NAME
FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
WHERE TABLE_NAME = 'last_selection'
  AND COLUMN_NAME = 'last_selection_id';
IF @ConstraintName IS NOT NULL
    EXEC ('ALTER TABLE last_selection DROP CONSTRAINT ' + @ConstraintName);

-- Add named last_selection constraints
ALTER TABLE last_selection
    ADD CONSTRAINT pk_last_selection PRIMARY KEY (last_selection_id);

-- --------------------

ALTER TABLE medusa_user
    ADD realm NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS;
ALTER TABLE medusa_admin
    ADD realm NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS;

-- --------------------

CREATE TABLE fido_user
(
    pk               BIGINT                                             NOT NULL IDENTITY (1, 1),
    iam_user_id      NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    user_handle      NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,
    relying_party_id NVARCHAR(255) COLLATE Latin1_General_CI_AI         NOT NULL,
    tenant_id        NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CS_AS  NOT NULL,

    CONSTRAINT pk_fido_user PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_user_handle
    ON fido_user (user_handle);
CREATE UNIQUE INDEX ux_fido_iam_user_rpid
    ON fido_user (iam_user_id, relying_party_id, tenant_id);

CREATE TABLE fido_credential
(
    id                            NVARCHAR(36) COLLATE SQL_Latin1_General_CP1_CS_AS NOT NULL,
    fido_user_pk                  BIGINT                                            NOT NULL,
    name                          NVARCHAR(100) COLLATE Latin1_General_CI_AI        NOT NULL,
    display_name                  NVARCHAR(100) COLLATE Latin1_General_CI_AI        NOT NULL,
    public_key_credential_id      VARBINARY(MAX)                                    NOT NULL,
    public_key_credential_id_hash VARBINARY(64)                                     NOT NULL,
    public_key                    VARBINARY(1024)                                   NOT NULL,
    signature_counter             BIGINT DEFAULT 0                                  NOT NULL,
    resident_requested            TINYINT                                           NOT NULL,
    attestation_object            VARBINARY(MAX)                                    NOT NULL,
    registered_at                 BIGINT                                            NOT NULL,
    first_used_at                 BIGINT,
    last_used_at                  BIGINT,
    total_usages                  INT    DEFAULT 0                                  NOT NULL,
    locked                        TINYINT                                           NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_fido_credential_user_pk
    ON fido_credential (fido_user_pk);
ALTER TABLE fido_credential
    ADD CONSTRAINT fk_fido_credential_user_pk FOREIGN KEY (fido_user_pk) REFERENCES fido_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_cred_id
    ON fido_credential (id);
CREATE UNIQUE INDEX ux_fido_cred_id_hash
    ON fido_credential (public_key_credential_id_hash);
