/*
 Migrates the Airlock IAM DB schema from version 7.1 to version 7.2.
 Written for Oracle DB.
 */

-- Comment out 'SET sqlblanklines on' if you're using a SQL client other than Oracle SQL*Plus
SET sqlblanklines on

ALTER SESSION SET NLS_LENGTH_SEMANTICS='CHAR';

ALTER TABLE account_link
    MODIFY username VARCHAR2(100);
ALTER TABLE account_link
    MODIFY provider_id VARCHAR2(20);
ALTER TABLE account_link
    MODIFY account_sub VARCHAR2(255);
ALTER TABLE account_link
    MODIFY account_info VARCHAR2(255);
ALTER TABLE account_link
    MODIFY tenant_id VARCHAR2(50);

ALTER TABLE user_consent
    MODIFY username VARCHAR2(100);
ALTER TABLE user_consent
    MODIFY consent_key VARCHAR2(50);
ALTER TABLE user_consent
    MODIFY state VARCHAR2(10);
ALTER TABLE user_consent
    MODIFY display_title VARCHAR2(255);
ALTER TABLE user_consent
    MODIFY display_text VARCHAR2(1000);
ALTER TABLE user_consent
    MODIFY tenant_id VARCHAR2(50);

CREATE TABLE subscription
(
    pk           INTEGER       NOT NULL,
    id           VARCHAR2(36)  NOT NULL,
    display_name VARCHAR2(100) NOT NULL,
    tenant_id    VARCHAR2(50)  NOT NULL,

    PRIMARY KEY (pk)
);
CREATE UNIQUE INDEX subscription_id_idx_unique
    ON subscription (id);
CREATE SEQUENCE subscription_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

ALTER TABLE principal
    MODIFY id VARCHAR2(36);
ALTER TABLE principal
    MODIFY display_name VARCHAR2(100);
ALTER TABLE principal
    MODIFY lock_reason VARCHAR2(100);
ALTER TABLE principal
    MODIFY tenant_id VARCHAR2(50);

ALTER TABLE principal
    ADD description VARCHAR2(100);
ALTER TABLE principal
    ADD label VARCHAR2(100);
ALTER TABLE principal
    ADD subscription_pk INTEGER;

ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);
CREATE INDEX subscription_idx
    ON principal (subscription_pk);

ALTER TABLE oauth2_attributes
    MODIFY authorization_server_id VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY client_id VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY client_secret VARCHAR2(450);
ALTER TABLE oauth2_attributes
    MODIFY token_endpoint_auth_method VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY grant_types VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY response_types VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY client_name VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY client_uri VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY logo_uri VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY scopes VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY tos_uri VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY policy_uri VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY jwks_uri VARCHAR2(200);
ALTER TABLE oauth2_attributes
    MODIFY software_id VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY software_version VARCHAR2(50);
ALTER TABLE oauth2_attributes
    MODIFY cert_issuer VARCHAR2(450);
ALTER TABLE oauth2_attributes
    MODIFY cert_subject VARCHAR2(450);
ALTER TABLE oauth2_attributes
    MODIFY tenant_id VARCHAR2(50);

ALTER TABLE oauth2_attribute_translation
    MODIFY attribute_name VARCHAR2(20);
ALTER TABLE oauth2_attribute_translation
    MODIFY locale VARCHAR2(20);
ALTER TABLE oauth2_attribute_translation
    MODIFY text VARCHAR2(200);

ALTER TABLE client_cert_credential
    MODIFY id VARCHAR2(36);
ALTER TABLE client_cert_credential
    MODIFY serial VARCHAR2(50);
ALTER TABLE client_cert_credential
    MODIFY issuer VARCHAR2(450);
ALTER TABLE client_cert_credential
    MODIFY subject VARCHAR2(450);
ALTER TABLE client_cert_credential
    MODIFY fingerprint VARCHAR2(200);
ALTER TABLE client_cert_credential
    MODIFY tenant_id VARCHAR2(50);

CREATE TABLE airlock_2fa_user
(
    pk                  INTEGER       NOT NULL,
    iam_user_id         VARCHAR2(100) NOT NULL,
    airlock_2fa_user_id VARCHAR2(50)  NOT NULL,
    tenant_id           VARCHAR2(50)  NOT NULL,

    PRIMARY KEY (pk)
);
CREATE SEQUENCE airlock_2fa_user_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;
CREATE UNIQUE INDEX a2fa_iam_user_id_idx_unique
    ON airlock_2fa_user (iam_user_id, tenant_id);
CREATE UNIQUE INDEX a2fa_user_id_idx_unique
    ON airlock_2fa_user (airlock_2fa_user_id);

CREATE TABLE airlock_2fa_activation_letter
(
    pk                  INTEGER       NOT NULL,
    id                  VARCHAR2(36)  NOT NULL,
    activation_code     VARCHAR2(750) NOT NULL,
    airlock_2fa_user_pk INTEGER       NOT NULL,
    creator             VARCHAR2(100) NOT NULL,

    PRIMARY KEY (pk),
    CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE
);
CREATE SEQUENCE a2fa_activation_letter_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;
CREATE UNIQUE INDEX a2fa_letter_id_idx_unique
    ON airlock_2fa_activation_letter (id);

CREATE TABLE api_key
(
    id            VARCHAR2(36)      NOT NULL,
    principal_pk  INTEGER           NOT NULL,
    fingerprint   VARCHAR2(30)      NOT NULL,
    secret        VARCHAR2(450)     NOT NULL,
    description   VARCHAR2(100),
    valid_from    TIMESTAMP,
    valid_to      TIMESTAMP,
    locked        INTEGER DEFAULT 0 NOT NULL,
    creation_date TIMESTAMP         NOT NULL,
    tenant_id     VARCHAR2(50)      NOT NULL,

    CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE
);
CREATE UNIQUE INDEX api_key_id_idx_unique
    ON api_key (id);

CREATE UNIQUE INDEX api_key_fp_idx_unique
    ON api_key (fingerprint, tenant_id);

CREATE TABLE plan_usage
(
    id              VARCHAR2(36)  NOT NULL,
    subscription_pk INTEGER       NOT NULL,
    name            VARCHAR2(100) NOT NULL,
    rate_limit      INTEGER,

    CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE
);
CREATE UNIQUE INDEX plan_usage_subscr_idx_unique
    ON plan_usage (name, subscription_pk);
CREATE UNIQUE INDEX plan_usage_id_idx_unique
    ON plan_usage (id);
