/*
 Migrates the Airlock IAM DB schema from version 7.2 to version 7.3.
 Written for Oracle DB.
 */

-- Comment out 'SET sqlblanklines on' if you're using a SQL client other than Oracle SQL*Plus
SET sqlblanklines on

ALTER SESSION SET NLS_LENGTH_SEMANTICS='CHAR';

alter table TOKEN
    DROP primary key cascade;
alter table TOKEN_ASSIGNMENT
    DROP primary key cascade;
DROP index TA_INDEX;

ALTER TABLE token
    ADD temp NUMBER(19);
ALTER TABLE token_assignment
    ADD temp NUMBER(19);

-- Update token.token_id
UPDATE token
SET temp = token_id
WHERE 1 = 1;

ALTER TABLE token
    MODIFY (token_id NULL);
update token
set token_id = null
WHERE 1 = 1;
alter table token
    modify token_id NUMBER(19);

UPDATE token
SET token_id = temp
WHERE 1 = 1;
ALTER TABLE token
    MODIFY (token_id NOT NULL);


-- Update token.obsoletes_token_id
UPDATE token
SET temp = obsoletes_token_id
WHERE 1 = 1;
update token
set obsoletes_token_id = null
WHERE 1 = 1;
alter table token
    modify obsoletes_token_id NUMBER(19);
UPDATE token
SET obsoletes_token_id = temp
WHERE 1 = 1;


-- Update token.activates_token_id
UPDATE token
SET temp = activates_token_id
WHERE 1 = 1;
update token
set activates_token_id = null
WHERE 1 = 1;
alter table token
    modify activates_token_id NUMBER(19);
UPDATE token
SET activates_token_id = temp
WHERE 1 = 1;


-- Update token.active
UPDATE token
SET temp = active
WHERE 1 = 1;
ALTER TABLE token
    MODIFY (active NULL);
update token
set active = null
WHERE 1 = 1;
alter table token
    modify active NUMBER(1);
UPDATE token
SET active = temp
WHERE 1 = 1;
ALTER TABLE token
    MODIFY (active NOT NULL);


-- Update token.total_usages
UPDATE token
SET temp = total_usages
WHERE 1 = 1;
ALTER TABLE token
    MODIFY (total_usages NULL);
update token
set total_usages = null
WHERE 1 = 1;
alter table token
    modify total_usages NUMBER(10);
UPDATE token
SET total_usages = temp
WHERE 1 = 1;
ALTER TABLE token
    MODIFY (total_usages NOT NULL);


-- Update token_assignment.ta_token_id
UPDATE token_assignment
SET temp = ta_token_id
WHERE 1 = 1;
ALTER TABLE token_assignment
    MODIFY (ta_token_id NULL);
update token_assignment
set ta_token_id = null
WHERE 1 = 1;
alter table token_assignment
    modify ta_token_id NUMBER(19);
UPDATE token_assignment
SET ta_token_id = temp
WHERE 1 = 1;
ALTER TABLE token_assignment
    MODIFY (ta_token_id NOT NULL);


-- Update token_assignment.ta_order_new
UPDATE token_assignment
SET temp = ta_order_new
WHERE 1 = 1;
ALTER TABLE token_assignment
    MODIFY (ta_order_new NULL);
update token_assignment
set ta_order_new = null
WHERE 1 = 1;
alter table token_assignment
    modify ta_order_new NUMBER(1);
UPDATE token_assignment
SET ta_order_new = temp
WHERE 1 = 1;
ALTER TABLE token_assignment
    MODIFY (ta_order_new NOT NULL);


-- cleanup
ALTER TABLE token
    DROP COLUMN temp;
ALTER TABLE token_assignment
    DROP COLUMN temp;

-- Restore PKs, constraints and indexes
ALTER TABLE token
    ADD CONSTRAINT pk_token_id PRIMARY KEY (token_id);
ALTER TABLE TOKEN
    ADD CONSTRAINT fk_obsoletes_token_id FOREIGN KEY (obsoletes_token_id) REFERENCES token (token_id);
ALTER TABLE TOKEN
    ADD CONSTRAINT fk_activates_token_id FOREIGN KEY (activates_token_id) REFERENCES token (token_id);
ALTER TABLE TOKEN_ASSIGNMENT
    ADD CONSTRAINT pk_ta_token_id_user PRIMARY KEY (TA_TOKEN_ID, TA_USER);
ALTER TABLE TOKEN_ASSIGNMENT
    ADD CONSTRAINT fk_ta_token_id FOREIGN KEY (ta_token_id) REFERENCES token (token_id);
CREATE INDEX ta_index ON token_assignment (ta_user, ta_token_id);

-- add new tables
CREATE TABLE oauth2_session
(
    pk NUMBER(19)   NOT NULL,
    id VARCHAR2(40) NOT NULL,

    PRIMARY KEY (pk)
);
CREATE SEQUENCE oauth2_session_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1;
CREATE UNIQUE INDEX ux_oauth2_session_id
    ON oauth2_session (id);

CREATE TABLE oauth2_session_attribute
(
    oauth2_session_pk NUMBER(19)    NOT NULL,
    name              VARCHAR2(50)  NOT NULL,
    value             VARCHAR2(200) NOT NULL,

    CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE
);
CREATE INDEX ix_osa_oauth2_session_pk
    ON oauth2_session_attribute (oauth2_session_pk);

CREATE SEQUENCE last_selection_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE last_selection
(
    last_selection_id INTEGER       NOT NULL,
    username          VARCHAR2(100) NOT NULL,
    option_id         VARCHAR2(50)  NOT NULL,
    selected_at       TIMESTAMP     NOT NULL,
    step_id           VARCHAR2(50)  NOT NULL,
    tenant_id         VARCHAR2(50)  NOT NULL,

    PRIMARY KEY (last_selection_id)
);
CREATE UNIQUE INDEX ux_last_selection
    ON last_selection (username, tenant_id, step_id);
