/*
 Migrates the Airlock IAM DB schema from version 7.3 to version 7.4.
 Written for Oracle DB.
 */

-- Comment out 'SET sqlblanklines on' if you're using a SQL client other than Oracle SQL*Plus
SET sqlblanklines on

ALTER SESSION SET NLS_LENGTH_SEMANTICS='CHAR';

-- Drop medusa_user constraints
ALTER TABLE medusa_user
    DROP PRIMARY KEY;

-- Add named medusa_user constraints
ALTER TABLE medusa_user
    ADD CONSTRAINT pk_medusa_user PRIMARY KEY (username);

-- --------------------

-- Drop medusa_admin constraints
ALTER TABLE medusa_admin
    DROP PRIMARY KEY;

-- Add named medusa_admin constraints
ALTER TABLE medusa_admin
    ADD CONSTRAINT pk_medusa_admin PRIMARY KEY (username);

-- --------------------

-- Drop medusa_maint_msg constraints
ALTER TABLE medusa_maint_msg
    DROP PRIMARY KEY;

-- Add named medusa_maint_msg constraints
ALTER TABLE medusa_maint_msg
    ADD CONSTRAINT pk_medusa_maint_msg PRIMARY KEY (name);

-- --------------------

-- Drop medusa_maint_msg_tnsl constraints
ALTER TABLE medusa_maint_msg_tnsl
    DROP PRIMARY KEY;

-- Add named medusa_maint_msg_tnsl constraints
ALTER TABLE medusa_maint_msg_tnsl
    ADD CONSTRAINT pk_medusa_maint_msg_tnsl PRIMARY KEY (message_ref, language);

-- --------------------

-- Drop medusa_pwd_selfservice_token constraints
ALTER TABLE medusa_pwd_selfservice_token
    DROP PRIMARY KEY;

-- Add named medusa_pwd_selfservice_token constraints
ALTER TABLE medusa_pwd_selfservice_token
    ADD CONSTRAINT pk_pwd_selfservice_token PRIMARY KEY (token_id);

-- --------------------

-- Drop login_history constraints
ALTER TABLE login_history
    DROP PRIMARY KEY;

-- Add named login_history constraints
ALTER TABLE login_history
    ADD CONSTRAINT pk_login_history PRIMARY KEY (id);

-- --------------------

-- Drop account_link constraints
ALTER TABLE account_link
    DROP PRIMARY KEY;

-- Add named account_link constraints
ALTER TABLE account_link
    ADD CONSTRAINT pk_account_link PRIMARY KEY (link_id);

-- --------------------

-- Drop user_consent constraints
ALTER TABLE user_consent
    DROP PRIMARY KEY;

-- Add named user_consent constraints
ALTER TABLE user_consent
    ADD CONSTRAINT pk_user_consent PRIMARY KEY (consent_id);

-- --------------------

-- Drop oauth2_attribute_translation constraints
ALTER TABLE oauth2_attribute_translation
    DROP CONSTRAINT fk_oauth2_attributes_pk;

-- Drop oauth2_attributes constraints
ALTER TABLE oauth2_attributes
    DROP PRIMARY KEY;
ALTER TABLE oauth2_attributes
    DROP CONSTRAINT fk_oa_principal_pk;

-- Drop api_key constraints
ALTER TABLE api_key
    DROP CONSTRAINT fk_api_key_principal_pk;

-- Drop client_cert_credential constraints
ALTER TABLE client_cert_credential
    DROP CONSTRAINT fk_ccc_principal_pk;

-- Drop principal constraints
ALTER TABLE principal
    DROP PRIMARY KEY;
ALTER TABLE principal
    DROP CONSTRAINT fk_subscription_pk;

-- Drop plan_usage constraints
ALTER TABLE plan_usage
    DROP CONSTRAINT fk_plan_usage_subscription_pk;

-- Drop subscription constraints
ALTER TABLE subscription
    DROP PRIMARY KEY;

-- Add named subscription constraints
ALTER TABLE subscription
    ADD CONSTRAINT pk_subscription PRIMARY KEY (pk);

-- Add named plan_usage constraints
DECLARE
    already_exists EXCEPTION;
    columns_indexed EXCEPTION;
    PRAGMA exception_init ( already_exists, -955 );
    PRAGMA exception_init (columns_indexed, -1408);
BEGIN
    EXECUTE IMMEDIATE 'CREATE INDEX fk_plan_usage_subscription_pk ON plan_usage (subscription_pk)';
EXCEPTION
    WHEN already_exists OR columns_indexed THEN NULL;
END;
/

ALTER TABLE plan_usage
    ADD CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE;

-- Add named principal constraints
ALTER TABLE principal
    ADD CONSTRAINT pk_principal PRIMARY KEY (pk);
ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);

-- Add named client_cert_credential constraints
ALTER TABLE client_cert_credential
    ADD CONSTRAINT fk_ccc_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named api_key constraints
DECLARE
    already_exists EXCEPTION;
    columns_indexed EXCEPTION;
    PRAGMA exception_init ( already_exists, -955 );
    PRAGMA exception_init (columns_indexed, -1408);
BEGIN
    EXECUTE IMMEDIATE 'CREATE INDEX fk_api_key_principal_pk ON api_key (principal_pk)';
EXCEPTION
    WHEN already_exists OR columns_indexed THEN NULL;
END;
/

ALTER TABLE api_key
    ADD CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attributes constraints
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT pk_oauth2_attributes PRIMARY KEY (pk);
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT fk_oa_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;

-- Add named oauth2_attribute_translation constraints
ALTER TABLE oauth2_attribute_translation
    ADD CONSTRAINT fk_oauth2_attributes_pk FOREIGN KEY (oauth2_attributes_pk) REFERENCES oauth2_attributes (pk) ON DELETE CASCADE;

-- --------------------

-- Drop oauth2_session_attribute constraints
ALTER TABLE oauth2_session_attribute
    DROP CONSTRAINT fk_oauth2_session_pk;

-- Drop oauth2_session constraints
ALTER TABLE oauth2_session
    DROP PRIMARY KEY;

-- Add named oauth2_session constraints
ALTER TABLE oauth2_session
    ADD CONSTRAINT pk_oauth2_session PRIMARY KEY (pk);

-- Add named oauth2_session_attribute constraints
ALTER TABLE oauth2_session_attribute
    ADD CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE;

-- --------------------

-- Drop airlock_2fa_activation_letter constraints
ALTER TABLE airlock_2fa_activation_letter
    DROP PRIMARY KEY;
ALTER TABLE airlock_2fa_activation_letter
    DROP CONSTRAINT fk_activation_letter_user_pk;

-- Drop airlock_2fa_user constraints
ALTER TABLE airlock_2fa_user
    DROP PRIMARY KEY;

-- Add named airlock_2fa_user constraints
ALTER TABLE airlock_2fa_user
    ADD CONSTRAINT pk_airlock_2fa_user PRIMARY KEY (pk);

-- Add named airlock_2fa_activation_letter constraints
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT pk_2fa_activation_letter PRIMARY KEY (pk);
DECLARE
    already_exists EXCEPTION;
    columns_indexed EXCEPTION;
    PRAGMA exception_init ( already_exists, -955 );
    PRAGMA exception_init (columns_indexed, -1408);
BEGIN
    EXECUTE IMMEDIATE 'CREATE INDEX fk_activation_letter_user_pk ON airlock_2fa_activation_letter (airlock_2fa_user_pk)';
EXCEPTION
    WHEN already_exists OR columns_indexed THEN NULL;
END;
/

ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE;

-- --------------------

-- Drop last_selection constraints
ALTER TABLE last_selection
    DROP PRIMARY KEY;

-- Add named last_selection constraints
ALTER TABLE last_selection
    ADD CONSTRAINT pk_last_selection PRIMARY KEY (last_selection_id);

-- --------------------

ALTER TABLE medusa_user
    ADD realm VARCHAR2(50);
ALTER TABLE medusa_admin
    ADD realm VARCHAR2(50);

-- --------------------

CREATE TABLE fido_user
(
    pk               NUMBER(19)    NOT NULL,
    iam_user_id      VARCHAR2(100) NOT NULL,
    user_handle      VARCHAR2(36)  NOT NULL,
    relying_party_id VARCHAR2(255) NOT NULL,
    tenant_id        VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_fido_user PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_user_handle
    ON fido_user (user_handle);
CREATE UNIQUE INDEX ux_fido_iam_user_rpid
    ON fido_user (iam_user_id, relying_party_id, tenant_id);
-- Sequence for pk
CREATE SEQUENCE fido_user_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE fido_credential
(
    id                            VARCHAR2(36)         NOT NULL,
    fido_user_pk                  NUMBER(19)           NOT NULL,
    name                          VARCHAR2(100)        NOT NULL,
    display_name                  VARCHAR2(100)        NOT NULL,
    public_key_credential_id      BLOB                 NOT NULL,
    public_key_credential_id_hash RAW(64)              NOT NULL,
    public_key                    RAW(1024)            NOT NULL,
    signature_counter             NUMBER(19) DEFAULT 0 NOT NULL,
    resident_requested            NUMBER(1)            NOT NULL,
    attestation_object            BLOB                 NOT NULL,
    registered_at                 NUMBER(19)           NOT NULL,
    first_used_at                 NUMBER(19),
    last_used_at                  NUMBER(19),
    total_usages                  INTEGER    DEFAULT 0 NOT NULL,
    locked                        NUMBER(1)            NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_fido_credential_user_pk
    ON fido_credential (fido_user_pk);
ALTER TABLE fido_credential
    ADD CONSTRAINT fk_fido_credential_user_pk FOREIGN KEY (fido_user_pk) REFERENCES fido_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_cred_id
    ON fido_credential (id);
CREATE UNIQUE INDEX ux_fido_cred_id_hash
    ON fido_credential (public_key_credential_id_hash);
