--
-- This script creates the Airlock IAM default schema for Oracle.
--

-- Comment out 'SET sqlblanklines on' if you're using a SQL client other than Oracle SQL*Plus
SET sqlblanklines on

-- Please ensure that the database uses the 'AL32UTF8' character encoding.
ALTER SESSION SET NLS_LENGTH_SEMANTICS='CHAR';

CREATE TABLE medusa_user
(
    -- General authentication information fields
    username                      VARCHAR2(100)     NOT NULL,
    auth_method                   VARCHAR2(50),
    next_auth_method              VARCHAR2(50),
    auth_migration_date           TIMESTAMP,
    locked                        INTEGER DEFAULT 0 NOT NULL,
    lock_reason                   VARCHAR2(50),
    lock_date                     TIMESTAMP,
    valid                         INTEGER DEFAULT 1 NOT NULL,
    not_valid_before              TIMESTAMP,
    not_valid_after               TIMESTAMP,

    -- Statistic data
    failed_logins                 INTEGER,
    failed_logins_before          INTEGER,
    failed_token_counts           CLOB,
    total_logins                  INTEGER,
    lat_succ_login                TIMESTAMP,
    lat_succ_login2               TIMESTAMP,
    lat_login_attempt             TIMESTAMP,
    first_login                   TIMESTAMP,
    unlock_attempts               INTEGER,
    lat_unlock_attempt            TIMESTAMP,

    -- self-registration data
    self_registered               INTEGER DEFAULT 0 NOT NULL,
    self_registration_date        TIMESTAMP,
    inactivity_reminder_sent_date TIMESTAMP,
    channel_verification_resends  INTEGER,

    -- Role settings
    roles                         VARCHAR2(1000),

    -- User context data
    salutation                    VARCHAR2(50),
    givenname                     VARCHAR2(50),
    surname                       VARCHAR2(50),
    street                        VARCHAR2(100),
    streetNumber                  VARCHAR2(20),
    address2                      VARCHAR2(100),
    zipcode                       VARCHAR2(12),
    town                          VARCHAR2(50),
    state                         VARCHAR2(50),
    country                       VARCHAR2(50),
    company                       VARCHAR2(75),
    email                         VARCHAR2(100),
    new_email                     VARCHAR2(100),
    language                      CHAR(2),
    birthdate                     DATE,
    disclaimer_tag                VARCHAR2(1024),
    remember_me_secret            VARCHAR2(1024),
    remember_me_gen_date          TIMESTAMP,
    realm                         VARCHAR2(50),

    -- Technical database fields
    rowVersionId                  INTEGER DEFAULT 0 NOT NULL,
    rowInsertDate                 TIMESTAMP,
    rowInsertUser                 VARCHAR2(20),
    rowUpdateDate                 TIMESTAMP,
    rowUpdateUser                 VARCHAR2(20),
    deleted                       INTEGER DEFAULT 0 NOT NULL,

    -- Password related fields
    pwd_hash                      VARCHAR2(4000),
    pwd_chg_enf                   INTEGER DEFAULT 0 NOT NULL,
    pwd_lat_del                   TIMESTAMP,
    pwd_lat_gen                   TIMESTAMP,
    pwd_lat_chg                   TIMESTAMP,
    pwd_next_chg                  TIMESTAMP,
    pwd_order_new                 INTEGER DEFAULT 0 NOT NULL,
    pwd_order_user                VARCHAR2(100),
    pwd_order_date                TIMESTAMP,
    pwd_failed_resets             INTEGER DEFAULT 0 NOT NULL,

    -- MTAN (SMS) related fields (including IAK)
    mtan_number                   VARCHAR2(25),
    mtan_del_date                 TIMESTAMP,
    mtan_ass_date                 TIMESTAMP,
    mtan_order_new                INTEGER DEFAULT 0 NOT NULL,
    mtan_order_user               VARCHAR2(100),
    mtan_order_date               TIMESTAMP,
    mtan_iak                      VARCHAR2(4000),

    -- Certificate related fields (including IAK and self registration)
    cert_subject_cn               VARCHAR2(200),
    cert_x509_data                VARCHAR2(4000),
    cert_serial                   VARCHAR2(20),
    cert_del_date                 TIMESTAMP,
    cert_ass_date                 TIMESTAMP,
    cert_order_new                INTEGER DEFAULT 0 NOT NULL,
    cert_order_user               VARCHAR2(100),
    cert_order_date               TIMESTAMP,
    cert_iak                      VARCHAR2(4000),

    -- OATH OTP related fields
    oathotp_data                  VARCHAR2(200),
    oathotp_serial                VARCHAR2(20),
    oathotp_del_date              TIMESTAMP,
    oathotp_gen_date              TIMESTAMP,
    oathotp_order_new             INTEGER DEFAULT 0 NOT NULL,
    oathotp_order_user            VARCHAR2(100),
    oathotp_order_date            TIMESTAMP,

    -- SecurID related fields
    securid_user                  VARCHAR2(25),
    securid_serial                VARCHAR2(20),
    securid_pin                   VARCHAR2(20),

    -- Matrix cards (TAN lists) related fields
    matrix_current_list           BLOB,
    matrix_next_list              BLOB,
    matrix_del_date               TIMESTAMP,
    matrix_gen_date               TIMESTAMP,
    matrix_order_new              INTEGER DEFAULT 0 NOT NULL,
    matrix_order_user             VARCHAR2(100),
    matrix_order_date             TIMESTAMP,
    matrix_chal_open_since        TIMESTAMP,
    matrix_open_chals             INTEGER DEFAULT 0 NOT NULL,

    -- Only one login possible
    last_gsid_date                TIMESTAMP,
    last_gsid_value               VARCHAR2(50),

    -- Secret questions features per user
    secret_questions_enabled      INTEGER DEFAULT 0 NOT NULL,

    CONSTRAINT pk_medusa_user PRIMARY KEY (username)
);

CREATE TABLE medusa_admin
(
    -- General authentication information fields
    username                     VARCHAR2(100)     NOT NULL,
    locked                       INTEGER DEFAULT 0 NOT NULL,
    lock_reason                  VARCHAR2(50),
    lock_date                    TIMESTAMP,
    not_valid_before             TIMESTAMP,
    not_valid_after              TIMESTAMP,

    -- Statistic data
    failed_logins                INTEGER,
    failed_logins_before         INTEGER,
    failed_first_factor_attempts INTEGER,
    failed_stepup_attempts       INTEGER,
    total_logins                 INTEGER,
    lat_succ_login               TIMESTAMP,
    lat_succ_login2              TIMESTAMP,
    lat_login_attempt            TIMESTAMP,
    first_login                  TIMESTAMP,

    -- Role settings
    roles                        VARCHAR2(1000),

    -- User context data
    givenname                    VARCHAR2(50),
    surname                      VARCHAR2(50),
    email                        VARCHAR2(100),
    realm                        VARCHAR2(50),

    -- Technical database fields
    rowVersionId                 INTEGER DEFAULT 0 NOT NULL,
    rowInsertDate                TIMESTAMP,
    rowInsertUser                VARCHAR2(20),
    rowUpdateDate                TIMESTAMP,
    rowUpdateUser                VARCHAR2(20),
    deleted                      INTEGER DEFAULT 0 NOT NULL,

    -- Password related fields
    pwd_hash                     VARCHAR2(4000),
    pwd_chg_enf                  INTEGER DEFAULT 0 NOT NULL,
    pwd_lat_del                  TIMESTAMP,
    pwd_lat_gen                  TIMESTAMP,
    pwd_lat_chg                  TIMESTAMP,
    pwd_next_chg                 TIMESTAMP,
    pwd_order_new                INTEGER DEFAULT 0 NOT NULL,

    CONSTRAINT pk_medusa_admin PRIMARY KEY (username)
);

CREATE TABLE medusa_maint_msg
(
    name             VARCHAR2(255)     NOT NULL,
    system_available INTEGER DEFAULT 1 NOT NULL,
    active           INTEGER DEFAULT 1 NOT NULL,
    valid_from       TIMESTAMP         NOT NULL,
    valid_to         TIMESTAMP         NOT NULL,
    location         VARCHAR2(50),
    -- Technical database fields
    rowVersionId     INTEGER DEFAULT 0 NOT NULL,
    rowInsertDate    TIMESTAMP,
    rowInsertUser    VARCHAR2(20),
    rowUpdateDate    TIMESTAMP,
    rowUpdateUser    VARCHAR2(20),
    deleted          INTEGER DEFAULT 0 NOT NULL,

    CONSTRAINT pk_medusa_maint_msg PRIMARY KEY (name)
);

CREATE TABLE medusa_maint_msg_tnsl
(
    message_ref   VARCHAR2(255)     NOT NULL,
    language      VARCHAR2(2)       NOT NULL,
    message       VARCHAR2(4000)    NOT NULL,
    -- Technical database fields
    rowVersionId  INTEGER DEFAULT 0 NOT NULL,
    rowInsertDate TIMESTAMP,
    rowInsertUser VARCHAR2(20),
    rowUpdateDate TIMESTAMP,
    rowUpdateUser VARCHAR2(20),
    deleted       INTEGER DEFAULT 0 NOT NULL,

    CONSTRAINT pk_medusa_maint_msg_tnsl PRIMARY KEY (message_ref, language)
);

CREATE TABLE token
(
    token_id                NUMBER(19)           NOT NULL,
    type                    VARCHAR2(20),
    serial_id               VARCHAR2(50),
    active                  NUMBER(1)  DEFAULT 0 NOT NULL,
    activation_date         NUMBER(19),
    obsoletes_token_id      NUMBER(19),
    validity_range_lower    NUMBER(19),
    validity_range_upper    NUMBER(19),
    generation_date         NUMBER(19),
    first_usage_date        NUMBER(19),
    latest_usage_date       NUMBER(19),
    total_usages            NUMBER(10) DEFAULT 0 NOT NULL,
    token_data              VARCHAR2(4000),
    activates_token_id      NUMBER(19),
    tracking_id             VARCHAR2(4000),
    generic_data_element_1  VARCHAR2(4000),
    generic_data_element_2  VARCHAR2(4000),
    generic_data_element_3  VARCHAR2(4000),
    generic_data_element_4  VARCHAR2(4000),
    generic_data_element_5  VARCHAR2(4000),
    generic_data_element_6  VARCHAR2(4000),
    generic_data_element_7  VARCHAR2(4000),
    generic_data_element_8  VARCHAR2(4000),
    generic_data_element_9  VARCHAR2(4000),
    generic_data_element_10 VARCHAR2(4000),
    generic_data_element_11 VARCHAR2(4000),
    generic_data_element_12 VARCHAR2(4000),

    CONSTRAINT pk_token_id PRIMARY KEY (token_id)
);
-- FOREIGN KEY indexes
CREATE INDEX obsoletes_token_id_idx
    ON token (obsoletes_token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_obsoletes_token_id FOREIGN KEY (obsoletes_token_id) REFERENCES token (token_id);
CREATE INDEX activates_token_id_index
    ON token (activates_token_id);
ALTER TABLE token
    ADD CONSTRAINT fk_activates_token_id FOREIGN KEY (activates_token_id) REFERENCES token (token_id);
-- Additional indexes
CREATE INDEX token_type_index
    ON token (type);
CREATE INDEX token_serial_id_index
    ON token (serial_id);
CREATE INDEX generic_data_element_1_idx
    ON token (generic_data_element_1);
CREATE INDEX generic_data_element_2_idx
    ON token (generic_data_element_2);
CREATE INDEX generic_data_element_3_idx
    ON token (generic_data_element_3);
CREATE INDEX generic_data_element_4_idx
    ON token (generic_data_element_4);
CREATE INDEX generic_data_element_6_idx
    ON token (generic_data_element_6);
CREATE INDEX generic_data_element_7_idx
    ON token (generic_data_element_7);
-- Sequence for token_id
CREATE SEQUENCE token_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE token_assignment
(
    ta_token_id               NUMBER(19)          NOT NULL,
    ta_user                   VARCHAR2(100)       NOT NULL,
    ta_assignment_date        NUMBER(19)          NOT NULL,
    ta_assignment_user        VARCHAR2(100)       NOT NULL,
    ta_order_new              NUMBER(1) DEFAULT 0 NOT NULL,
    ta_order_new_date         NUMBER(19),
    ta_order_new_user         VARCHAR2(100),
    ta_order_options          VARCHAR2(250),
    ta_additional_information VARCHAR2(250),
    ta_comment                VARCHAR2(4000),

    CONSTRAINT pk_ta_token_id_user PRIMARY KEY (ta_token_id, ta_user)
);
-- FOREIGN KEY indexes
ALTER TABLE token_assignment
    ADD CONSTRAINT fk_ta_token_id FOREIGN KEY (ta_token_id) REFERENCES token (token_id) ON DELETE CASCADE;
-- Additional indexes
CREATE INDEX ta_index
    ON token_assignment (ta_user, ta_token_id);

CREATE TABLE login_history
(
    id                     INTEGER       NOT NULL,
    username               VARCHAR2(100) NOT NULL,
    login_timestamp        TIMESTAMP     NOT NULL,
    ip                     VARCHAR2(45)  NOT NULL,
    header_user_agent      VARCHAR2(512),
    header_accept_encoding VARCHAR2(512),
    header_accept_language VARCHAR2(512),
    geoloc_present         INTEGER       NOT NULL,
    geoloc_continent       CHAR(2),
    geoloc_country         CHAR(2),
    geoloc_subdivision     CHAR(3),
    geoloc_city            VARCHAR2(256),
    geoloc_zip             VARCHAR2(32),
    geoloc_timezone        VARCHAR2(128),
    geoloc_latitude        VARCHAR2(16),
    geoloc_longitude       VARCHAR2(16),

    CONSTRAINT pk_login_history PRIMARY KEY (id)
);
-- Additional indexes
CREATE INDEX login_history_idx
    ON login_history (username);
-- Sequence for id
CREATE SEQUENCE history_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE account_link
(
    link_id        INTEGER       NOT NULL,
    username       VARCHAR2(100) NOT NULL,
    provider_id    VARCHAR2(20)  NOT NULL,
    account_sub    VARCHAR2(255) NOT NULL,
    account_info   VARCHAR2(255),
    established_at TIMESTAMP     NOT NULL,
    tenant_id      VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_account_link PRIMARY KEY (link_id)
);
-- Additional indexes
CREATE INDEX account_link_idx_username
    ON account_link (username, tenant_id);
CREATE INDEX account_link_idx_usr_prov
    ON account_link (username, provider_id, tenant_id);
CREATE UNIQUE INDEX account_link_idx_unique
    ON account_link (account_sub, provider_id, tenant_id);
-- Sequence for link_id
CREATE SEQUENCE account_link_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE subscription
(
    pk           INTEGER       NOT NULL,
    id           VARCHAR2(36)  NOT NULL,
    display_name VARCHAR2(100) NOT NULL,
    tenant_id    VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_subscription PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX subscription_id_idx_unique
    ON subscription (id);
-- Sequence for pk
CREATE SEQUENCE subscription_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE principal
(
    pk              INTEGER           NOT NULL,
    id              VARCHAR2(36)      NOT NULL,
    display_name    VARCHAR2(100)     NOT NULL,
    locked          INTEGER DEFAULT 0 NOT NULL,
    lock_reason     VARCHAR2(100),
    lock_date       TIMESTAMP,
    creation_date   TIMESTAMP         NOT NULL,
    tenant_id       VARCHAR2(50)      NOT NULL,
    description     VARCHAR2(100),
    label           VARCHAR2(100),
    subscription_pk INTEGER,

    CONSTRAINT pk_principal PRIMARY KEY (pk)
);
-- FOREIGN KEY indexes
CREATE INDEX subscription_idx
    ON principal (subscription_pk);
ALTER TABLE principal
    ADD CONSTRAINT fk_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk);
-- Additional indexes
CREATE UNIQUE INDEX principal_id_idx_unique
    ON principal (id);
CREATE INDEX tenant_id_idx
    ON principal (tenant_id);
-- Sequence for pk
CREATE SEQUENCE principal_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE oauth2_attributes
(
    pk                         INTEGER      NOT NULL,
    principal_pk               INTEGER      NOT NULL,
    authorization_server_id    VARCHAR2(50) NOT NULL,
    client_id                  VARCHAR2(50) NOT NULL,
    client_secret              VARCHAR2(450),
    client_id_issued_at        TIMESTAMP    NOT NULL,
    client_secret_expires_at   TIMESTAMP,
    redirect_uris              CLOB,
    token_endpoint_auth_method VARCHAR2(50),
    grant_types                VARCHAR2(200),
    response_types             VARCHAR2(50),
    client_name                VARCHAR2(200),
    client_uri                 VARCHAR2(200),
    logo_uri                   VARCHAR2(200),
    scopes                     VARCHAR2(200),
    contacts                   CLOB,
    tos_uri                    VARCHAR2(200),
    policy_uri                 VARCHAR2(200),
    jwks_uri                   VARCHAR2(200),
    software_id                VARCHAR2(50),
    software_version           VARCHAR2(50),
    cert_issuer                VARCHAR2(450),
    cert_subject               VARCHAR2(450),
    tenant_id                  VARCHAR2(50) NOT NULL,

    CONSTRAINT pk_oauth2_attributes PRIMARY KEY (pk)
);
-- FOREIGN KEY indexes
CREATE UNIQUE INDEX oa2_attrs_principal_idx_unique
    ON oauth2_attributes (principal_pk);
ALTER TABLE oauth2_attributes
    ADD CONSTRAINT fk_oa_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX oa2_attrs_as_cid_idx_unique
    ON oauth2_attributes (authorization_server_id, client_id, tenant_id);
-- Sequence for pk
CREATE SEQUENCE oauth2_attributes_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE oauth2_attribute_translation
(
    oauth2_attributes_pk INTEGER NOT NULL,
    attribute_name       VARCHAR2(20),
    locale               VARCHAR2(20),
    text                 VARCHAR2(200)
);
-- FOREIGN KEY indexes
CREATE INDEX oauth2_attributes_pk_idx
    ON oauth2_attribute_translation (oauth2_attributes_pk);
ALTER TABLE oauth2_attribute_translation
    ADD CONSTRAINT fk_oauth2_attributes_pk FOREIGN KEY (oauth2_attributes_pk) REFERENCES oauth2_attributes (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX oauth2_translation_idx_unique
    ON oauth2_attribute_translation (oauth2_attributes_pk, attribute_name, locale);

CREATE TABLE oauth2_session
(
    pk NUMBER(19)   NOT NULL,
    id VARCHAR2(40) NOT NULL,

    CONSTRAINT pk_oauth2_session PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_oauth2_session_id
    ON oauth2_session (id);
-- Sequence for pk
CREATE SEQUENCE oauth2_session_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1;

CREATE TABLE oauth2_session_attribute
(
    oauth2_session_pk NUMBER(19)    NOT NULL,
    name              VARCHAR2(50)  NOT NULL,
    content           VARCHAR2(200) NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_osa_oauth2_session_pk
    ON oauth2_session_attribute (oauth2_session_pk);
ALTER TABLE oauth2_session_attribute
    ADD CONSTRAINT fk_oauth2_session_pk FOREIGN KEY (oauth2_session_pk) REFERENCES oauth2_session (pk) ON DELETE CASCADE;

CREATE TABLE client_cert_credential
(
    id           VARCHAR2(36)  NOT NULL,
    principal_pk INTEGER       NOT NULL,
    serial       VARCHAR2(50)  NOT NULL,
    issuer       VARCHAR2(450) NOT NULL,
    subject      VARCHAR2(450) NOT NULL,
    not_before   TIMESTAMP     NOT NULL,
    not_after    TIMESTAMP     NOT NULL,
    certificate  CLOB          NOT NULL,
    fingerprint  VARCHAR2(200) NOT NULL,
    tenant_id    VARCHAR2(50)  NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX principal_pk_idx
    ON client_cert_credential (principal_pk);
ALTER TABLE client_cert_credential
    ADD CONSTRAINT fk_ccc_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ccc_id_idx_unique
    ON client_cert_credential (id);
CREATE UNIQUE INDEX fingerprint_idx_unique
    ON client_cert_credential (fingerprint, tenant_id);
CREATE INDEX ccc_subject_idx
    ON client_cert_credential (subject);

CREATE TABLE airlock_2fa_user
(
    pk                  INTEGER       NOT NULL,
    iam_user_id         VARCHAR2(100) NOT NULL,
    airlock_2fa_user_id VARCHAR2(50)  NOT NULL,
    tenant_id           VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_airlock_2fa_user PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX a2fa_iam_user_id_idx_unique
    ON airlock_2fa_user (iam_user_id, tenant_id);
CREATE UNIQUE INDEX a2fa_user_id_idx_unique
    ON airlock_2fa_user (airlock_2fa_user_id);
-- Sequence for pk
CREATE SEQUENCE airlock_2fa_user_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE airlock_2fa_activation_letter
(
    pk                  INTEGER       NOT NULL,
    id                  VARCHAR2(36)  NOT NULL,
    activation_code     VARCHAR2(750) NOT NULL,
    airlock_2fa_user_pk INTEGER       NOT NULL,
    creator             VARCHAR2(100) NOT NULL,

    CONSTRAINT pk_2fa_activation_letter PRIMARY KEY (pk)
);
-- FOREIGN KEY indexes
CREATE INDEX fk_activation_letter_user_pk
    ON airlock_2fa_activation_letter (airlock_2fa_user_pk);
ALTER TABLE airlock_2fa_activation_letter
    ADD CONSTRAINT fk_activation_letter_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX a2fa_letter_id_idx_unique
    ON airlock_2fa_activation_letter (id);
-- Sequence for pk
CREATE SEQUENCE a2fa_activation_letter_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE airlock_2fa_activation_order
(
    id                  VARCHAR2(36)  NOT NULL,
    airlock_2fa_user_pk INTEGER       NOT NULL,
    creator             VARCHAR2(100) NOT NULL,
    ordered_at          NUMBER(19)    NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_a2fa_activ_order_user_pk
    ON airlock_2fa_activation_order (airlock_2fa_user_pk);
ALTER TABLE airlock_2fa_activation_order
    ADD CONSTRAINT fk_a2fa_activ_order_user_pk FOREIGN KEY (airlock_2fa_user_pk) REFERENCES airlock_2fa_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_a2fa_activ_order_id
    ON airlock_2fa_activation_order (id);

CREATE TABLE api_key
(
    id            VARCHAR2(36)      NOT NULL,
    principal_pk  INTEGER           NOT NULL,
    fingerprint   VARCHAR2(30)      NOT NULL,
    secret        VARCHAR2(450)     NOT NULL,
    description   VARCHAR2(100),
    valid_from    TIMESTAMP,
    valid_to      TIMESTAMP,
    locked        INTEGER DEFAULT 0 NOT NULL,
    creation_date TIMESTAMP         NOT NULL,
    tenant_id     VARCHAR2(50)      NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX fk_api_key_principal_pk
    ON api_key (principal_pk);
ALTER TABLE api_key
    ADD CONSTRAINT fk_api_key_principal_pk FOREIGN KEY (principal_pk) REFERENCES principal (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX api_key_id_idx_unique
    ON api_key (id);
CREATE UNIQUE INDEX api_key_fp_idx_unique
    ON api_key (fingerprint, tenant_id);

CREATE TABLE plan_usage
(
    id              VARCHAR2(36)  NOT NULL,
    subscription_pk INTEGER       NOT NULL,
    name            VARCHAR2(100) NOT NULL,
    rate_limit      INTEGER
);
-- FOREIGN KEY indexes
CREATE INDEX fk_plan_usage_subscription_pk
    ON plan_usage (subscription_pk);
ALTER TABLE plan_usage
    ADD CONSTRAINT fk_plan_usage_subscription_pk FOREIGN KEY (subscription_pk) REFERENCES subscription (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX plan_usage_subscr_idx_unique
    ON plan_usage (name, subscription_pk);
CREATE UNIQUE INDEX plan_usage_id_idx_unique
    ON plan_usage (id);

CREATE TABLE last_selection
(
    last_selection_id INTEGER       NOT NULL,
    username          VARCHAR2(100) NOT NULL,
    option_id         VARCHAR2(50)  NOT NULL,
    selected_at       TIMESTAMP     NOT NULL,
    step_id           VARCHAR2(50)  NOT NULL,
    tenant_id         VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_last_selection PRIMARY KEY (last_selection_id)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_last_selection
    ON last_selection (username, tenant_id, step_id);
-- Sequence for last_selection_id
CREATE SEQUENCE last_selection_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE fido_user
(
    pk               NUMBER(19)    NOT NULL,
    iam_user_id      VARCHAR2(100) NOT NULL,
    user_handle      VARCHAR2(36)  NOT NULL,
    relying_party_id VARCHAR2(255) NOT NULL,
    tenant_id        VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_fido_user PRIMARY KEY (pk)
);
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_user_handle
    ON fido_user (user_handle);
CREATE UNIQUE INDEX ux_fido_iam_user_rpid
    ON fido_user (iam_user_id, relying_party_id, tenant_id);
-- Sequence for pk
CREATE SEQUENCE fido_user_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

CREATE TABLE fido_credential
(
    id                            VARCHAR2(36)         NOT NULL,
    fido_user_pk                  NUMBER(19)           NOT NULL,
    name                          VARCHAR2(100)        NOT NULL,
    display_name                  VARCHAR2(100)        NOT NULL,
    public_key_credential_id      BLOB                 NOT NULL,
    public_key_credential_id_hash RAW(64)              NOT NULL,
    public_key                    RAW(1024)            NOT NULL,
    signature_counter             NUMBER(19) DEFAULT 0 NOT NULL,
    resident_requested            NUMBER(1)            NOT NULL,
    attestation_object            BLOB                 NOT NULL,
    registered_at                 NUMBER(19)           NOT NULL,
    first_used_at                 NUMBER(19),
    last_used_at                  NUMBER(19),
    total_usages                  INTEGER    DEFAULT 0 NOT NULL,
    locked                        NUMBER(1)            NOT NULL
);
-- FOREIGN KEY indexes
CREATE INDEX ix_fido_credential_user_pk
    ON fido_credential (fido_user_pk);
ALTER TABLE fido_credential
    ADD CONSTRAINT fk_fido_credential_user_pk FOREIGN KEY (fido_user_pk) REFERENCES fido_user (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_fido_cred_id
    ON fido_credential (id);
CREATE UNIQUE INDEX ux_fido_cred_id_hash
    ON fido_credential (public_key_credential_id_hash);

CREATE TABLE remember_me_token
(
    pk          NUMBER(19)    NOT NULL,
    id          VARCHAR2(36)  NOT NULL,
    iam_user_id VARCHAR2(100) NOT NULL,
    token_hash  RAW(64)       NOT NULL,
    created_at  NUMBER(19)    NOT NULL,
    renewed_at  NUMBER(19),
    tenant_id   VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_remember_me_token PRIMARY KEY (pk)
);
-- Sequence for pk
CREATE SEQUENCE remember_me_token_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

-- Additional indexes
CREATE INDEX ix_remember_me_iam_user_id on remember_me_token (iam_user_id);
CREATE INDEX ix_remember_me_token_created_at on remember_me_token (created_at, tenant_id);
CREATE INDEX ix_remember_me_token_renewed_at on remember_me_token (renewed_at, tenant_id);
CREATE UNIQUE INDEX ux_remember_me_id ON remember_me_token (id, tenant_id);
CREATE UNIQUE INDEX ux_remember_me_token_hash on remember_me_token (token_hash, tenant_id);

CREATE TABLE remember_me_request_metadata
(
    pk                      NUMBER(19)   NOT NULL,
    remember_me_token_pk    NUMBER(19)   NOT NULL,
    request_type            NUMBER(1)    NOT NULL,
    request_ip              VARCHAR2(45) NOT NULL,
    header_user_agent       VARCHAR2(4000),
    geolocation_continent   VARCHAR2(2),
    geolocation_country     VARCHAR2(2),
    geolocation_subdivision VARCHAR2(3),
    geolocation_city        VARCHAR2(256),
    geolocation_zip         VARCHAR2(32),
    geolocation_timezone    VARCHAR2(128),
    geolocation_latitude    VARCHAR2(16),
    geolocation_longitude   VARCHAR2(16),

    CONSTRAINT pk_remember_me_request_metadata PRIMARY KEY (pk)
);
-- Sequence for pk
CREATE SEQUENCE remember_me_request_metadata_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

-- FOREIGN KEY indexes
ALTER TABLE remember_me_request_metadata
    ADD CONSTRAINT fk_remember_me_token_pk FOREIGN KEY (remember_me_token_pk) REFERENCES remember_me_token (pk) ON DELETE CASCADE;
-- Additional indexes
CREATE UNIQUE INDEX ux_remember_me_token_request_type ON remember_me_request_metadata (remember_me_token_pk, request_type);

CREATE TABLE flow_continuation_token
(
    pk                    NUMBER(19)    NOT NULL,
    token_hash            VARCHAR2(86)  NOT NULL,
    user_id               VARCHAR2(100) NOT NULL,
    target_flow_id        VARCHAR2(30)  NOT NULL,
    created_at            NUMBER(19)    NOT NULL,
    valid_until           NUMBER(19)    NOT NULL,
    configuration_context VARCHAR2(50)  NOT NULL,
    forward_url           VARCHAR2(200),
    tenant_id             VARCHAR2(50)  NOT NULL,

    CONSTRAINT pk_flow_continuation_token PRIMARY KEY (pk)
);
-- Sequence for pk
CREATE SEQUENCE flow_continuation_token_seq
    MINVALUE 1
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

-- Additional indexes
CREATE UNIQUE INDEX ux_flow_continuation_token ON flow_continuation_token (token_hash);

CREATE TABLE user_trail_log
(
    pk                    NUMBER(19) GENERATED ALWAYS AS IDENTITY,
    created_at            NUMBER(19)     NOT NULL,
    log_level             VARCHAR2(8)    NOT NULL,
    session_id            VARCHAR2(256)  NOT NULL,
    request_id            VARCHAR2(256)  NOT NULL,
    configuration_context VARCHAR2(50)   NOT NULL,
    client_ip             VARCHAR2(45)   NOT NULL,
    target_id             VARCHAR2(100)  NOT NULL,
    authentee_id          VARCHAR2(100)  NOT NULL,
    authentee_provided_id VARCHAR2(100)  NOT NULL,
    message               VARCHAR2(4000) NOT NULL,
    tenant_id             VARCHAR2(50)   NOT NULL,

    CONSTRAINT pk_user_trail_log PRIMARY KEY (pk)
);

-- Additional indexes
CREATE INDEX ix_user_trail_log_target_id ON user_trail_log (target_id, created_at);
CREATE INDEX ix_user_trail_log_created_at ON user_trail_log (created_at);

CREATE TABLE device_usage
(
    user_id           VARCHAR2(100) NOT NULL,
    device_type       VARCHAR2(20)  NOT NULL,
    device_id         VARCHAR2(50)  NOT NULL,
    total_usages      INTEGER       NOT NULL,
    first_usage_date  NUMBER(19)    NOT NULL,
    latest_usage_date NUMBER(19)    NOT NULL,
    tenant_id         VARCHAR2(50)  NOT NULL
);

-- Additional indexes
CREATE UNIQUE INDEX ux_device_usage ON device_usage (device_id, user_id, device_type, tenant_id);

CREATE TABLE accepted_sso_tickets
(
    ticket_id  VARCHAR(100) NOT NULL,
    expires_at NUMBER(19)   NOT NULL,
    tenant_id  VARCHAR2(50) NOT NULL
);
-- Additional indexes
CREATE UNIQUE INDEX ux_accepted_sso_tickets ON accepted_sso_tickets (ticket_id, tenant_id);
CREATE INDEX ix_accepted_sso_tickets_expires_at ON accepted_sso_tickets (expires_at);

CREATE TABLE oauth2_consent
(
    id                      VARCHAR2(36)  NOT NULL,
    user_id                 VARCHAR2(100) NOT NULL,
    authorization_server_id VARCHAR2(50)  NOT NULL,
    client_id               VARCHAR2(50)  NOT NULL,
    scope_name              VARCHAR2(50)  NOT NULL,
    status                  VARCHAR2(20)  NOT NULL,
    updated_at              NUMBER(19)    NOT NULL,
    tenant_id               VARCHAR2(50)  NOT NULL
);
-- Additional indexes
CREATE UNIQUE INDEX ux_oauth2_consent_id
    ON oauth2_consent (id);

CREATE INDEX ix_oauth2_consent_user_as_client
    ON oauth2_consent (user_id, authorization_server_id, client_id);
